"""User Functions.

The following functions give you access to view and edit users in the
Gateway.
"""

from __future__ import print_function

__all__ = [
    "addCompositeSchedule",
    "addHoliday",
    "addRole",
    "addSchedule",
    "addUser",
    "createScheduleAdjustment",
    "editHoliday",
    "editRole",
    "editSchedule",
    "editUser",
    "getHoliday",
    "getHolidayNames",
    "getHolidays",
    "getNewUser",
    "getRoles",
    "getSchedule",
    "getScheduleNames",
    "getScheduledUsers",
    "getSchedules",
    "getUser",
    "getUsers",
    "isUserScheduled",
    "removeHoliday",
    "removeRole",
    "removeSchedule",
    "removeUser",
]

import system.date
from com.inductiveautomation.ignition.client.util.gui.scheduling import (
    ScheduleModel,
)
from com.inductiveautomation.ignition.common.messages import UIResponse
from com.inductiveautomation.ignition.common.user import PyUser
from com.inductiveautomation.ignition.common.user.schedule import HolidayModel
from java.util import Date, Locale


def addCompositeSchedule(name, scheduleOne, scheduleTwo, description=None):
    """Allows two schedules to be combined into a composite schedule.

    Args:
        name (str): The name of the new composite schedule.
        scheduleOne (str): The first schedule to combine.
        scheduleTwo (str): The second schedule to combine.
        description (str): Description of the new combined schedule.
            Optional.

    Returns:
        UIResponse: A UIResponse object with lists of warnings, errors,
            and info about the success or failure of the add.
    """
    print(name, scheduleOne, scheduleTwo, description)
    return UIResponse(Locale.ENGLISH)


def addHoliday(holiday):
    """Allows a holiday to be added.

    Args:
        holiday (HolidayModel): The holiday to add, as a HolidayModel
            object.

    Returns:
        UIResponse: A UIResponse object with lists of warnings, errors,
            and info about the success or failure of the add.
    """
    print(holiday)
    return UIResponse(Locale.ENGLISH)


def addRole(userSource, role):
    """Allows a role to the specified user source.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource (str): The user source to add a role to. Blank will
            use the default user source.
        role (str): The role to add. Role must not be blank and must not
            already exist.

    Returns:
        UIResponse: A UIResponse object with lists of warnings, errors,
            and info about the success or failure of the add.
    """
    print(userSource, role)
    return UIResponse(Locale.ENGLISH)


def addSchedule(schedule):
    """Allows a schedule to be added.

    Args:
        schedule (ScheduleModel): The schedule to add. Can be a
            BasicScheduleModel or CompositeScheduleModel object (or any
            other class that extends AbstractScheduleModel).

    Returns:
        UIResponse: A UIResponse object with lists of warnings, errors,
            and info about the success or failure of the add.
    """
    print(schedule)
    return UIResponse(Locale.ENGLISH)


def addUser(userSource, user):
    """Adds a new user to a user source.

    Used in combination with getNewUser to create new user.

    Args:
        userSource (str):
        user (PyUser): The user to add, as a User object. Refer also to
            the PyUser class.

    Returns:
        UIResponse: A UIResponse object with lists of warnings, errors,
            and info about the success or failure of the add attempt.
    """
    print(userSource, user)
    return UIResponse(Locale.ENGLISH)


def createScheduleAdjustment(startDate, endDate, isAvailable, note):
    """Creates a schedule adjustment.

    Args:
        startDate (Date): The starting date of the schedule
            adjustment.
        endDate (Date): The ending date of the schedule adjustment.
        isAvailable (bool): True if the user is available during this
            schedule adjustment.
        note (str): A note about the schedule adjustment.

    Returns:
        ScheduleAdjustment: A ScheduleAdjustment object that can be
            added to a user.
    """
    print(startDate, endDate, isAvailable, note)


def editHoliday(holidayName, holiday):
    """Allows a holiday to be edited.

    Args:
        holidayName (str): The name of the holiday to edit. Name is
            case-sensitive.
        holiday (HolidayModel): The edited holiday, as a HolidayModel
            object.

    Returns:
        UIResponse: An object with lists of warnings, errors, and info
            about the success or failure of the edit.
    """
    print(holidayName, holiday)
    return UIResponse(Locale.ENGLISH)


def editRole(userSource, oldName, newName):
    """Renames a role in the specified user source.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource (str): The user source in which the role is found.
            Blank will use the default user source.
        oldName (str): The role to edit. Role must not be blank and must
            exist.
        newName (str): The new name for the role. Must not be blank.

    Returns:
        UIResponse: An object with lists of warnings, errors, and info
            about the success or failure of the edit.
    """
    print(userSource, oldName, newName)
    return UIResponse(Locale.ENGLISH)


def editSchedule(scheduleName, schedule):
    """Allows a schedule to be edited.

    Args:
        scheduleName (str): The name of the schedule to edit. Name is
            case-sensitive.
        schedule (ScheduleModel): The schedule to add. Can be a
            BasicScheduleModel or CompositeScheduleModel object (or any
            other class that extends AbstractScheduleModel).

    Returns:
        UIResponse: An object with lists of warnings, errors, and info
            about the success or failure of the edit.
    """
    print(scheduleName, schedule)
    return UIResponse(Locale.ENGLISH)


def editUser(userSource, user):
    """Alters a specific user in a user source, replacing the previous
    data with the new data passed in.

    Args:
        userSource (str): The user source in which the user is found.
            Blank will use the default user source.
        user (PyUser): The user to update, as a User object. Refer also
            to the PyUser class.

    Returns:
        UIResponse: A UIResponse object with lists of warnings, errors,
            and information returned after the edit attempt.
    """
    print(userSource, user)
    return UIResponse(Locale.ENGLISH)


def getHoliday(holidayName):
    """Returns a specific holiday.

    Args:
        holidayName (str): The name of the holiday to return.
            Case-sensitive.

    Returns:
        HolidayModel: The holiday, as a HolidayModel object, or None if
            not found.
    """
    print(holidayName)


def getHolidayNames():
    """Returns a collection of Strings of all holiday names.

    Returns:
        list[str]: A list of all holiday names, or an empty list if no
            holidays are defined.
    """
    return ["Labor Day", "Groundhog Day"]


def getHolidays():
    """Returns a sequence of all of the holidays available.

    Returns:
        list[HolidayModel]: A list of holidays, as HolidayModel objects.
    """
    return []


def getNewUser(userSource, username):
    """Creates a new user object.

    The user will not be added to the user source until addUser is
    called.

    Args:
        userSource (str): The name of the user source in which to create
            a user.
        username (str): The username for the new user. Does not check if
            username already exists or is valid.

    Returns:
        PyUser: The new user, as a User object. Refer also to the PyUser
            class.
    """
    print(userSource, username)
    return PyUser()


def getRoles(userSource):
    """Returns a sequence of strings representing all of the roles
    configured in a specific user source.

    Args:
        userSource (str): The user source to fetch the roles for.

    Returns:
        list[str]: A List of Strings that holds all the roles in the
            user source.
    """
    print(userSource)


def getSchedule(scheduleName):
    """Returns a specific schedule.

    Args:
        scheduleName (str): The name of the schedule to return.
            Case-sensitive.

    Returns:
         ScheduleModel: The schedule, which can be a BasicScheduleModel
            object, CompositeScheduleModel object, or another type
            registered by a module. If a schedule was not found, the
            function will return None if called from a Vision Client or
            the Designer. if called in from a Perspective Session or
            anywhere else in the Gateway scope, will throw an
            IllegalArgumentException.
    """
    print(scheduleName)


def getScheduleNames():
    """Returns a sequence of strings representing the names of all of
    the schedules available.

    Returns:
        list[str]: A List of Strings that holds the names of all the
            available schedules.
    """
    return ["A", "Always", "B", "C", "Example", "MyComposite", "MySchedule"]


def getScheduledUsers(userSource, date=None):
    """Returns a list of users that are scheduled on.

    If no users are scheduled, it will return an empty list.

    Args:
        userSource (str): The name of the user source to check for
            scheduled users.
        date (object): The date to check schedules for. May be a Java
            Date or Unix Time in ms.. If omitted, the current date and
            time will be used. Optional.

    Returns:
        list[PyUser]: List of all Users scheduled for the given date,
            taking schedule adjustments into account. Refer also to the
            PyUser class.
    """
    date = system.date.now() if date is None else date
    print(userSource, date)


def getSchedules():
    """Returns a sequence of all available schedule models, which can be
    used to return configuration information on the schedule, such as
    time for each day of the week.

    Returns:
        list[ScheduleModel]: A list of schedules. Each schedule can be a
            BasicScheduleModel object, CompositeScheduleModel object, or
            another type registered by a module.
    """
    return [ScheduleModel()]


def getUser(userSource, username):
    """Looks up a specific user in a user source, by username.

    The full User object is returned except for the user's password.

    Args:
        userSource (str): The name of the user source to search for the
            user in. Can be a blank string to use the Vision Client's
            default user source.
        username (str): The username of the user to search for.

    Returns:
        PyUser: The user, as a User object. Refer also to the PyUser
            class.
    """
    print(userSource, username)
    return PyUser()


def getUsers(userSource):
    """Retrieves the list of users in a specific user source.

    The User objects that are returned contain all of the information
    about that user, except for the user's password.

    Args:
        userSource (str): The name of the user source to find the users
            in.

    Returns:
        list[PyUser]: A list of User objects. Refer also to the PyUser
            class.
    """
    print(userSource)
    # You may return more than one User object.
    return [PyUser()]


def isUserScheduled(user, date=None):
    """Will check if a specified User is scheduled currently or on a
    specified date/time.

    Args:
        user (PyUser): The user object to check the schedule for.
        date (object): The date to check schedules for. May be a Java
            Date or Unix Time in ms. If omitted, the current date and
            time will be used. Optional.

    Returns:
        bool: True if the user is scheduled for the specified date,
            False if not.
    """
    date = system.date.now() if date is None else date
    print(user, date)
    return True


def removeHoliday(holidayName):
    """Allows a holiday to be deleted.

    Args:
        holidayName (str): The name of the holiday to delete. Name is
            case-sensitive.

    Returns:
        UIResponse: An object with lists of warnings, errors, and info
            about the success or failure of the deletion.
    """
    print(holidayName)
    return UIResponse(Locale.ENGLISH)


def removeRole(userSource, role):
    """Removes a role from the specified user source.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource (str): The user source in which the role is found.
            Blank will use the default user source.
        role (str): The role to remove. The role must exist.

    Returns:
        UIResponse: An object with lists of warnings, errors, and info
            about the success or failure of the deletion.
    """
    print(userSource, role)
    return UIResponse(Locale.ENGLISH)


def removeSchedule(scheduleName):
    """Allows a schedule to be deleted.

    Note that schedules which are used in Composite Schedules can not be
    deleted until they are removed from the Composite Schedule.

    Args:
        scheduleName (str): The name of the schedule to delete. Name is
            case-sensitive.

    Returns:
        UIResponse: An object with lists of warnings, errors, and info
            about the success or failure of the deletion.
    """
    print(scheduleName)
    return UIResponse(Locale.ENGLISH)


def removeUser(userSource, username):
    """Removes a specific user from the a user source based on username.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource (str): The user source in which the user is found.
            Blank will use the default user source.
        username (str): The username of the user to remove.

    Returns:
        UIResponse: An UIResponse object with lists of warnings, errors,
            and information returned after the removal attempt.
    """
    print(userSource, username)



__all__ = ["AlarmQueryResult", "AlarmQueryResultImpl"]


class AlarmQueryResult(object):
    """This is the result of a query against the alarming system, for
    both status and history.

    It provides the results as a list, but also provides additional
    helper functions for getting the event and associated data as a
    dataset.
    """

    def getAssociatedDate(self, uuid):
        raise NotImplementedError

    def getDataSet(self):
        raise NotImplementedError

    def getEvent(self, uuid):
        raise NotImplementedError


class AlarmQueryResultImpl(AlarmQueryResult):
    def __init__(self, *args):
        pass

    def getAssociatedDate(self, uuid):
        pass

    def getDataSet(self):
        pass

    def getEvent(self, uuid):
        pass



__all__ = ["BasicQualifiedValue", "QualityCode", "QualifiedValue"]

from java.lang import Object
from java.util import Date


class QualityCode(Object):
    """QualityCode contains a 32-bit integer code and optionally a
    diagnostic string.
    """

    Bad = None
    Bad_AccessDenied = None
    Bad_AggregateNotFound = None
    Bad_DatabaseNotConnected = None
    Bad_Disabled = None
    Bad_Failure = None
    Bad_GatewayCommOff = None
    Bad_LicenseExceeded = None
    Bad_NotConnected = None
    Bad_NotFound = None
    Bad_OutOfRange = 524
    Bad_ReadOnly = None
    Bad_Stale = None
    Bad_ReferenceNotFound = None
    Bad_TrialExpired = None
    Bad_Unauthorized = None
    Bad_Unsupported = None
    Error = None
    Error_Configuration = None
    Error_CycleDetected = None
    Error_DatabaseQuery = None
    Error_Exception = None
    Error_ExpressionEval = None
    Error_Formatting = None
    Error_InvalidPathSyntax = None
    Error_IO = None
    Error_ScriptEval = None
    Error_TagExecution = None
    Error_TimeoutExpired = None
    Error_TypeConversion = None
    Good = None
    Good_Backfill = None
    Good_Initial = None
    Good_Overload = None
    Good_Provisional = None
    Good_Unspecified = None
    Good_WritePending = None
    Uncertain = None
    Uncertain_LastKnownValue = None
    Uncertain_InitialValue = None
    Uncertain_DataSubNormal = None
    Uncertain_EngineeringUnitsExceeded = None
    Uncertain_IncompleteOperation = None

    def __init__(self, *args):
        pass

    def derive(self, diagnosticMessage):
        pass

    def getCode(self):
        pass

    def getDiagnosticMessage(self):
        pass

    def getLevel(self):
        pass

    def getName(self):
        pass

    def isBad(self):
        pass

    def isBadOrError(self):
        pass

    def isError(self):
        pass

    def isGood(self):
        pass

    def isNotGood(self):
        pass

    def isUncertain(self):
        pass

    class Level(Object):
        def __init__(self):
            pass

        def code(self, userCode):
            pass

        @staticmethod
        def valueOf(name):
            pass

        @staticmethod
        def values():
            pass


class QualifiedValue(object):
    """Represents a value with a DataQuality & timestamp attached to
    it.
    """

    def getQuality(self):
        raise NotImplementedError

    def getTimestamp(self):
        raise NotImplementedError

    def getValue(self):
        raise NotImplementedError


class BasicQualifiedValue(QualifiedValue, Object):
    quality = QualityCode.Bad_Stale
    timestamp = Date()
    value = None

    def __init__(self, *arg):
        pass

    @staticmethod
    def fromObject(obj):
        pass

    def getQuality(self):
        return self.quality

    def getTimestamp(self):
        return self.timestamp

    def getValue(self):
        return self.value

    def setQuality(self, code):
        self.quality = code

    def setTimestamp(self, timestamp):
        self.timestamp = timestamp

    def setValue(self, value):
        self.value = value

    @staticmethod
    def unwrap(obj):
        pass

    @staticmethod
    def updateTimestamp(copy):
        pass

    @staticmethod
    def valueOf(obj):
        pass



"""Transaction Group Functions.

The following functions give you access to import and remove Transaction
Groups.
"""

from __future__ import print_function

__all__ = ["loadFromFile", "removeGroups"]


def loadFromFile(filePath, projectName, mode):
    """Loads a transaction group configuration from an xml export, into
    the specified project (creating the project if necessary). The mode
    parameter dictates how overwrites occur.

    Args:
        filePath (str): The path to a valid transaction group xml or csv
            file.
        projectName (str): The name of the project to load into.
        mode (int): How duplicates will be handled. 0 = Overwrite,
            1 = Ignore, 2 = Replace the existing project with this one.
    """
    print(filePath, projectName, mode)


def removeGroups(projectName, paths):
    """Removes the specified groups from the project. The group paths
    are "Folder/Path/To/GroupName", separated by forward slashes.

    Args:
        projectName (str): The project to remove from. If the project
            does not exist, throws an IllegalArgumentException.
        paths (list[str]): A collection of paths to remove. The group
            paths are "Folder/Path/To/GroupName", separated by forward
            slashes.
    """
    print(projectName, paths)



__all__ = ["Aggregate", "AggregateInfo"]

from java.lang import Object


class Aggregate(object):
    """This interface defines an aggregation function used by the
    history query system.

    Different types of history providers may support different Aggregate
    functions, and may define new types of aggregates. The name and
    description are for informational purposes, aggregates are only
    identified by their id (name and description should not be taken
    into account).

    The general implementation class is AggregateInfo. Common or "well
    known" aggregates are defined in the AggregationMode enum. The
    system works like this for historical reasons, previous to 7.7 only
    the AggregationMode aggregates were used. After, with the
    introduction of history providers as an extension point, new
    providers could define any aggregation function.
    """

    def getDesc(self):
        raise NotImplementedError

    def getId(self):
        raise NotImplementedError

    def getName(self):
        raise NotImplementedError


class AggregateInfo(Object, Aggregate):
    def getDesc(self):
        pass

    def getId(self):
        pass

    def getName(self):
        pass



"""BACnet Functions.

The following functions are used with the BACnet driver and a BACnet/IP
device.
"""

from __future__ import print_function

__all__ = ["synchronizeTime", "synchronizeTimeUtc", "writeWithPriority"]


def synchronizeTime(deviceName):
    """Notifies the remote device of the correct current time, which is
    the system time (factoring in timezone and DST) of the server
    Ignition is running on.

    Args:
        deviceName (str): The name of the configured BACnet/IP device
            instance to write from.
    """
    print(deviceName)


def synchronizeTimeUtc(deviceName):
    """Notifies the remote device of the correct current time in UTC.

    Args:
        deviceName (str): The name of the configured BACnet/IP device
            instance to write from.
    """
    print(deviceName)


def writeWithPriority(deviceName, objectType, objectId, value, priority):
    """Write to the Present_Value attribute of an object with a custom
    priority level.

    Args:
        deviceName (str): The name of the configured BACnet/IP device
            instance to write from.
        objectType (int): The numeric id of the objectType of the object
            instance being written to. See the objectType reference
            table below.
        objectId (int): The object instance number to write to.
        value (object): The value to write. Clearing a value can be
            accomplished by writing a None value.
        priority (int): The priority level to write the value at. Must
            be in the range [1...16].
    """
    print(deviceName, objectType, objectId, value, priority)



"""Provides the classes for implementing networking applications."""

from __future__ import print_function

__all__ = ["InetSocketAddress", "Socket", "SocketAddress"]

from java.lang import Object


class Socket(Object):
    def __init__(self, *args):
        pass

    def bind(self, bindpoint):
        pass

    def close(self):
        pass

    def connect(self, endpoint, timeout):
        pass

    def getChannel(self):
        pass

    def getInetAddress(self):
        pass

    def getInputStream(self):
        pass

    def getKeepAlive(self):
        pass

    def getLocalAddress(self):
        pass

    def getLocalPort(self):
        pass

    def getLocalSocketAddress(self):
        pass

    def getOOBInline(self):
        pass

    def getOption(self, name):
        pass

    def getOutputStream(self):
        pass

    def getPort(self):
        pass

    def getReceiveBufferSize(self):
        pass

    def getRemoteSocketAddress(self):
        pass

    def getReuseAddress(self):
        pass

    def getSendBufferSize(self):
        pass

    def getSoLinger(self):
        pass

    def getSoTimeout(self):
        pass

    def getTcpNoDelay(self):
        pass

    def getTrafficClass(self):
        pass

    def isBound(self):
        pass

    def isClosed(self):
        pass

    def isConnected(self):
        pass

    def isInputShutdown(self):
        pass

    def isOutputShutdown(self):
        pass

    def sendUrgentData(self, data):
        pass

    def setKeepAlive(self, on):
        pass

    def setOOBInline(self, on):
        pass

    def setOption(self, name, value):
        pass

    def setPerformancePreferences(self, connectionTime, latency, bandwidth):
        pass

    def setReceiveBufferSize(self, size):
        pass

    def setReuseAddress(self, on):
        pass

    def setSendBufferSize(self, size):
        pass

    @staticmethod
    def setSocketImplFactory(fac):
        pass

    def setSoLinger(self, on, linger):
        pass

    def setSoTimeout(self, timeout):
        pass

    def setTcpNoDelay(self, on):
        pass

    def setTrafficClass(self, tc):
        pass

    def shutdownInput(self):
        pass

    def shutdownOutput(self):
        pass

    def supportedOptions(self):
        pass


class SocketAddress(Object):
    def __init__(self):
        pass


class InetSocketAddress(SocketAddress):
    def __init__(self, *args):
        print(args)
        super(InetSocketAddress, self).__init__()

    @staticmethod
    def createUnresolved(host, port):
        return InetSocketAddress(host, port)

    def getAddress(self):
        pass

    def getHostname(self):
        pass

    def getHostString(self):
        pass

    def getPort(self):
        pass

    def isUnresolved(self):
        pass



__all__ = ["LoggerEx"]

from java.lang import Object


class LoggerEx(Object):
    """This class is a wrapper around a logger which provides additional
    useful tools.

    To create one, use the newBuilder() function and configure the
    builder.
    """

    def __init__(self):
        pass

    def getLogger(self):
        pass

    def getName(self):
        pass

    @staticmethod
    def log(*args):
        pass



"""Twilio Functions.

The following functions give you access to read info and send SMS
through Twilio. This requires the Twilio Module, which is not included
in a typical install.
"""

from __future__ import print_function

__all__ = [
    "getAccounts",
    "getAccountsDataset",
    "getPhoneNumbers",
    "getPhoneNumbersDataset",
    "sendSms",
]

from com.inductiveautomation.ignition.common import BasicDataset


def getAccounts():
    """Return a list of Twilio accounts that have been configured in the
    Gateway.

    Returns:
        list[str]: A list of configured Twilio accounts.
    """
    return ["twilio_account1", "twilio_account2"]


def getAccountsDataset():
    """Return a list of Twilio accounts that have been configured in the
    Gateway as a single-column Dataset.

    Returns:
        BasicDataset: A list of configured Twilio accounts as a
            single-column Dataset.
    """
    return BasicDataset()


def getPhoneNumbers(accountName):
    """Returns a list of outgoing phone numbers for a Twilio account.

    Note that these numbers are supplied by Twilio, and are not defined
    on a user in Ignition.

    Args:
        accountName (str): The Twilio account to retrieve phone numbers
            for.

    Returns:
        list[str]: A list of phone numbers for the given Twilio account.
    """
    phoneNumbers = []
    if accountName == "Jenny":
        phoneNumbers.append("+12058675309")
    return phoneNumbers


def getPhoneNumbersDataset(accountName):
    """Return a list of outgoing phone numbers for a Twilio account as a
    single-column Dataset.

    Note that these numbers are supplied by Twilio, and are not defined
    on a user in Ignition.

    Args:
        accountName (str): The Twilio account to retrieve phone numbers
            for.

    Returns:
        BasicDataset: A list of phone numbers for the given Twilio
            account as a single-column Dataset.
    """
    print(accountName)
    return BasicDataset()


def sendSms(accountName, fromNumber, toNumber, message):
    """Sends an SMS message.

    Args:
        accountName (str): The Twilio account to send the SMS from.
        fromNumber (str): The outbound phone number belonging to the
            Twilio account to use.
        toNumber (str): The phone number of the recipient.
        message (str): The body of the SMS.
    """
    print(accountName, fromNumber, toNumber, message)



__all__ = ["MessageInterface", "MessageReceiver", "UIResponse"]

from java.lang import Object


class MessageInterface(object):
    def addMessageReceiver(self, protocol, rcv):
        raise NotImplementedError

    def sendCall(self, protocol, scope, msg):
        raise NotImplementedError

    def sendMessage(self, protocol, scope, msg):
        raise NotImplementedError


class MessageReceiver(object):
    def receiveCall(self, msg):
        raise NotImplementedError


class UIResponse(Object):
    def __init__(self, locale):
        self.locale = locale

    def attempt(self, method):
        pass

    def error(self, message, args):
        pass

    def getErrors(self):
        pass

    def getInfos(self):
        pass

    def getLocale(self):
        pass

    def getWarns(self):
        pass

    def info(self, message, args):
        pass

    def warn(self, message, args):
        pass

    def wrap(self, locale, fx):
        pass



__all__ = ["HolidayModel", "ScheduleAdjustment"]

from java.lang import Object


class HolidayModel(Object):
    def __init__(self, name, date, repeatAnnually):
        self.name = name
        self.date = date
        self.repeatAnnually = repeatAnnually

    def getDate(self):
        return self.date

    def getName(self):
        return self.name

    def isRepeatedAnnually(self):
        return self.repeatAnnually


class ScheduleAdjustment(Object):
    def __init__(self):
        pass

    def contains(self, timestamp):
        pass

    def getEnd(self):
        pass

    def getNote(self):
        pass

    def getStart(self):
        pass

    def isAvailable(self):
        pass

    def setAvailable(self, available):
        pass

    def setEnd(self, end):
        pass

    def setNoe(self, note):
        pass

    def setStart(self, start):
        pass



from __future__ import print_function

__all__ = ["DatasetUtilities", "SProcCall", "SystemUtilities"]

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.script.abc import (
    AbstractJythonSequence,
)
from com.inductiveautomation.ignition.common.script.message import Request
from java.lang import Object
from java.util import Locale


class DatasetUtilities(Object):
    @staticmethod
    def addColumn(*args):
        pass

    @staticmethod
    def addRow(*args):
        pass

    @staticmethod
    def addRows(*args):
        pass

    @staticmethod
    def appendDataset(ds1, ds2):
        pass

    @staticmethod
    def clearDataset(ds):
        pass

    @staticmethod
    def dataSetToExcel(headerRow, datasets):
        pass

    @staticmethod
    def dataSetToExcelBytes(headerRow, objects, nullsEmpty, sheetNames):
        pass

    @staticmethod
    def dataSetToExcelStreaming(headerRow, objects, out, nullsEmpty):
        pass

    @staticmethod
    def dataSetToHTML(headerRow, ds, title):
        pass

    @staticmethod
    def dataSetToHTMLStreaming(headerRow, ds, title, fw):
        pass

    @staticmethod
    def deleteRow(ds, row):
        pass

    @staticmethod
    def deleteRows(ds, rows):
        pass

    @staticmethod
    def filterColumns(dataset, columns):
        pass

    @staticmethod
    def formatDates(dataset, format, locale=Locale.US):
        pass

    @staticmethod
    def fromCSV(csv):
        pass

    @staticmethod
    def fromCSVJava(csv):
        pass

    @staticmethod
    def getColumnHeaders(ds):
        pass

    @staticmethod
    def insertColumn(*args):
        pass

    @staticmethod
    def insertRow(*args):
        pass

    @staticmethod
    def setValue(*args):
        pass

    @staticmethod
    def sort(ds, keyColumn, ascending=None, naturalOrdering=None):
        pass

    @staticmethod
    def toCSV(*args, **kwargs):
        pass

    @staticmethod
    def toCSVJava(ds, showHeaders, forExport, localized=False):
        pass

    @staticmethod
    def toCSVJavaStreaming(ds, showHeaders, forExport, sw, localized):
        pass

    @staticmethod
    def toDataSet(*args):
        pass

    @staticmethod
    def toExcel(*args, **kwargs):
        pass

    @staticmethod
    def toJSONObject(data):
        pass

    @staticmethod
    def toPyDataSet(dataset):
        pass

    @staticmethod
    def updateRow(ds, row, changes):
        pass

    class PyDataSet(Dataset, AbstractJythonSequence):
        _ds = None

        def __init__(self, ds=None):
            self._ds = ds

        def getColumnCount(self):
            pass

        def getColumnIndex(self, name):
            pass

        def getColumnName(self, col):
            pass

        def getColumnNames(self):
            pass

        def getColumnType(self, col):
            pass

        def getColumnTypes(self):
            pass

        def getPrimitiveValueAt(self, row, col):
            pass

        def getQualityAt(self, row, col):
            pass

        def getRowCount(self):
            pass

        def getValueAt(self, row, col):
            pass


class SProcCall(Object):
    def __init__(self):
        pass

    def getResultSet(self):
        """Returns a dataset that is the resulting data of the stored
        procedure, if any.

        Returns:
            BasicDataset: The dataset that is the resulting data of the
                stored procedure, if any.
        """
        print(self)
        return BasicDataset()

    def getUpdateCount(self):
        """Returns the number of rows modified by the stored procedure,
        or -1 if not applicable.

        Returns:
             int: The number of rows modified by the stored procedure,
                or -1 if not applicable.
        """
        print(self)
        return 1

    def getReturnValue(self):
        """Returns the return value, if registerReturnParam had been
        called.

        Returns:
             int: The return value, if registerReturnParam had been
                called.
        """
        print(self)
        return 0

    def getOutParamValue(self, param):
        """Returns the value of the previously registered out-parameter.

        Args:
            param (object): Index (int) or name (str) of the previously
                registered out-parameter.

        Returns:
            object: The value of the previously registered
                out-parameter.
        """
        print(self, param)
        return 0

    def registerInParam(self, param, typeCode, value):
        """Registers an in parameter for the stored procedure.

        Args:
            param (object): Index (int starting at 1, not 0), or name
                (str).
            typeCode (int): Type code constant.
            value (object): Value of type typeCode.
        """
        print(self, param, typeCode, value)

    def registerOutParam(self, param, typeCode):
        """Registers an out parameter for the stored procedure.

        Args:
            param (object): Index (int starting at 1, not 0), or name
                (str).
            typeCode (int): Type code constant.
        """
        print(self, param, typeCode)

    def registerReturnParam(self, typeCode):
        """Use this function to specify the datatype of the returned
        value.

        Args:
            typeCode (int): Type code constant.
        """
        print(self, typeCode)

    class SProcArg(Object):
        def getParamType(self):
            pass

        def getValue(self):
            pass

        def isInParam(self):
            pass

        def isOutParam(self):
            pass

        def setParamType(self, paramType):
            pass

        def setValue(self, value):
            pass

        def toString(self):
            pass

    class SProcArgKey(Object):
        def getParamIndex(self):
            pass

        def getParamName(self):
            pass

        def hashCode(self):
            pass

        def isNamedParam(self):
            pass

        def toString(self):
            pass


class SystemUtilities(Object):
    @staticmethod
    def logger(loggerName):
        pass

    @staticmethod
    def parseTranslateArguments(*args, **kwargs):
        pass

    class RequestImpl(Object, Request):
        def __init__(self, timeout):
            self.timeout = timeout

        def checkTimeout(self):
            pass

        def dispatchFunc(self):
            pass

        def finishExceptionally(self, e):
            pass

        def finishSuccessfully(self, value):
            pass

        def getLongId(self):
            pass

        def cancel(self):
            pass

        def get(self):
            pass

        def getError(self):
            pass

        def onError(self, func):
            pass

        def onSuccess(self, func):
            pass



from __future__ import print_function

__all__ = [
    "INavUtilities",
    "NavUtilities",
    "PrintUtilities",
    "WindowUtilities",
]

from java.awt.image import BufferedImage
from java.lang import Object


class INavUtilities(object):
    """Parent interface to coordinate the functions between NavUtilities
    and NavUtilitiesDispatcher.
    """

    def centerWindow(self, arg):
        raise NotImplementedError

    def closeParentWindow(self, event):
        raise NotImplementedError

    def closeWindow(self, arg):
        raise NotImplementedError

    def getCurrentWindow(self):
        raise NotImplementedError

    def goBack(self):
        raise NotImplementedError

    def goForward(self):
        raise NotImplementedError

    def goHome(self):
        raise NotImplementedError

    def openWindow(self, *args):
        raise NotImplementedError

    def openWindowImpl(self, path, params, openAdditional):
        raise NotImplementedError

    def openWindowInstance(self, *args):
        raise NotImplementedError

    def swapTo(self, *args):
        raise NotImplementedError

    def swapWindow(self, *args):
        raise NotImplementedError


class NavUtilities(INavUtilities):
    def centerWindow(self, arg):
        pass

    def closeParentWindow(self, event):
        pass

    def closeWindow(self, arg):
        pass

    def getCurrentWindow(self):
        pass

    def goBack(self):
        pass

    def goForward(self):
        pass

    def goHome(self):
        pass

    def openWindow(self, *args):
        pass

    def openWindowImpl(self, path, params, openAdditional):
        pass

    def openWindowInstance(self, *args):
        pass

    def swapTo(self, *args):
        pass

    def swapWindow(self, *args):
        pass


class PrintUtilities(Object):
    def __init__(self, app):
        print(self, app)

    def createImage(self, c):
        print(self, c)
        width = height = imageType = 1
        return BufferedImage(width, height, imageType)

    def createPrintJob(self, c):
        pass

    def printToImage(self, c, fileName=None):
        pass

    class JythonPrintJob(Object):
        def getBottomMargin(self):
            pass

        def getLeftMargin(self):
            pass

        def getOrientation(self):
            pass

        def getPageHeight(self):
            pass

        def getPageWidth(self):
            pass

        def getPrinterName(self):
            pass

        def getRightMargin(self):
            pass

        def getTopMargin(self):
            pass

        def getZoomFactor(self):
            pass

        def isFitToPage(self):
            pass

        def isShowPrintDialog(self):
            pass

        def setBottomMargin(self, bottomMargin):
            pass

        def setFitToPage(self, fitToPage):
            pass

        def setLeftMargin(self, leftMargin):
            pass

        def setMargins(self, m):
            pass

        def setOrientation(self, orientation):
            pass

        def setPageHeight(self, pageHeight):
            pass

        def setPageWidth(self, pageWidth):
            pass

        def setPrinterName(self, printerName):
            pass

        def setRightMargin(self, rightMargin):
            pass

        def setShowPrintDialog(self, showPrintDialog):
            pass

        def setZoomFactor(self, zoomFactor):
            pass


class WindowUtilities(Object):
    """These are the scripting functions mounted at system.gui.*.
    Changes to this class must be made carefully, as some of the true
    implementations actually reside in the subclass,
    WindowUtilitiesForDesktop.
    """

    def confirm(self, *args):
        pass

    def errorBox(self, *args):
        pass

    def inputBox(self, *args):
        pass

    def messageBox(self, *args):
        pass

    def passwordBox(self, *args):
        pass

    def warningBox(self, *args):
        pass



__all__ = ["FPMIWindow"]

from javax.swing import JInternalFrame


class FPMIWindow(JInternalFrame):
    CACHE_ALWAYS = 2
    CACHE_AUTO = 0
    CACHE_NEVER = 1
    DOCK_EAST = 2
    DOCK_FLOAT = 0
    DOCK_NORTH = 2
    DOCK_SOUTH = 4
    DOCK_WEST = 3
    PARENT_WINDOW_NAME = "_parent"
    SHOW_ALWAYS = 0
    SHOW_NEVER = 1
    SHOW_MAXIMIZED = 2

    _path = "Path/To/Window"

    def __init__(self, name):
        self.name = name

    def getPath(self):
        return self._path

    def getRootContainer(self):
        print(self)



__all__ = ["JythonHttpClient"]

from java.lang import Object


class JythonHttpClient(Object):
    """A Jython-optimized wrapper around the base HttpClient available
    in Java 11+.

    Mostly, through convenience functions that make things easier to use
    from Jython.
    """

    def delete(self, *args, **kwargs):
        pass

    def deleteAsync(self, *args, **kwargs):
        pass

    def get(self, *args, **kwargs):
        pass

    def getAsync(self, *args, **kwargs):
        pass

    def getConnectTimeout(self):
        pass

    def getCookieManager(self):
        pass

    def getJavaClient(self):
        pass

    def getRedirectPolicy(self):
        pass

    def head(self, *args, **kwargs):
        pass

    def headAsync(self, *args, **kwargs):
        pass

    def options(self, *args, **kwargs):
        pass

    def parseChart(self, contentType):
        pass

    def patch(self, *args, **kwargs):
        pass

    def patchAsync(self, *args, **kwargs):
        pass

    def post(self, *args, **kwargs):
        pass

    def postAsync(self, *args, **kwargs):
        pass

    def put(self, *args, **kwargs):
        pass

    def putAsync(self, *args, **kwargs):
        pass

    def request(self, *args, **kwargs):
        pass

    def requestAsync(self, *args, **kwargs):
        pass

    def setGson(self, gson):
        pass

    def trace(self, *args, **kwargs):
        pass

    def traceAsync(self, *args, **kwargs):
        pass



"""Utility Functions.

The following functions give you access to view various Gateway and
Client data, as well as interact with other various systems.
"""

from __future__ import print_function

__all__ = [
    "audit",
    "beep",
    "execute",
    "exit",
    "getAvailableLocales",
    "getAvailableTerms",
    "getClientId",
    "getConnectTimeout",
    "getConnectionMode",
    "getEdition",
    "getGatewayAddress",
    "getGatewayStatus",
    "getGlobals",
    "getInactivitySeconds",
    "getLocale",
    "getLogger",
    "getProjectName",
    "getProperty",
    "getReadTimeout",
    "getSessionInfo",
    "getSystemFlags",
    "getVersion",
    "invokeAsynchronous",
    "invokeLater",
    "jsonDecode",
    "jsonEncode",
    "modifyTranslation",
    "playSoundClip",
    "queryAuditLog",
    "retarget",
    "sendMessage",
    "sendRequest",
    "sendRequestAsync",
    "setConnectTimeout",
    "setConnectionMode",
    "setLocale",
    "setLoggingLevel",
    "setReadTimeout",
    "threadDump",
    "translate",
]

import getpass
import json
import os
import platform
import sys

import system.__version__ as version
import system.date
import system.security
from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.ignition.common.model import Version
from com.inductiveautomation.ignition.common.script.builtin import (
    DatasetUtilities,
    SystemUtilities,
)
from com.inductiveautomation.ignition.common.script.message import Request
from com.inductiveautomation.ignition.common.util import LoggerEx
from java.awt import Toolkit
from java.lang import Thread
from java.util import Date


def audit(
    action=None,
    actionValue=None,
    auditProfile="",
    actor=None,
    actorHost="localhost",
    originatingSystem=None,
    eventTimestamp=None,
    originatingContext=4,
    statusCode=0,
):
    """Inserts a record into an audit profile.

    Args:
        action (str): What happened. Default is null. Optional.
        actionValue (str): What the action happened to. Default is null.
            Optional.
        auditProfile (str): Where the audit record should be stored.
            Defaults to the project's audit profile (if specified), or
            the gateway audit profile if calling in the gateway or
            perspective scope. Optional.
        actor (str): Who made the change. Will be populated
            automatically if omitted, assuming there is a known user.
            Optional.
        actorHost (str): The hostname of whoever made the change. Will
            be populated automatically if omitted.
        originatingSystem (object): An even-length list providing
            additional context to the audit event. Optional.
        eventTimestamp (Date): When the event happened. Will be set
            to the current time if omitted. Optional.
        originatingContext (int): What scope the event originated from:
            1 means Gateway, 2 means Designer, 4 means Client. Will be
            set automatically if omitted. Optional.
        statusCode (int): A quality code to attach to the object.
            Defaults to 0, indicating no special meaning. Optional.
    """
    actor = system.security.getUsername() if actor is None else actor
    eventTimestamp = (
        system.date.now() if eventTimestamp is None else eventTimestamp
    )
    print(
        action,
        actionValue,
        auditProfile,
        actor,
        actorHost,
        originatingSystem,
        eventTimestamp,
        originatingContext,
        statusCode,
    )


def beep():
    """Tells the computer to make a "beep" sound."""
    platforms = {
        "linux1": "Linux",
        "linux2": "Linux",
        "darwin": "macOS",
        "win32": "Windows",
    }

    if "java" in sys.platform:
        Toolkit.getDefaultToolkit().beep()
    elif sys.platform in platforms:
        if platforms[sys.platform] == "Windows":
            try:
                import winsound

                winsound.MessageBeep(winsound.MB_ICONEXCLAMATION)
            except ImportError:
                print("Beep!")
        elif platforms[sys.platform] == "macOS":
            os.system('say "beep"')
        elif platforms[sys.platform] == "Linux":
            print("\a")
    else:
        print("Beep!")


def execute(commands):
    """Executes the given commands via the operating system, in a
    separate process.

    The commands argument is an array of strings. The first string is
    the program to execute, with subsequent strings being the arguments
    to that command.

    Args:
        commands (list[str]): A list containing the command (1st entry)
            and associated arguments (remaining entries) to execute.
    """
    print(commands)


def exit(force=False):
    """Exits the running client, as long as the shutdown intercept
    script doesn't cancel the shutdown event.

    Set force to True to not give the shutdown intercept script a chance
    to cancel the exit. Note that this will quit the Client completely.
    You can use system.security.logout() to return to the login screen.

    Args:
        force (bool): If True (1), the shutdown-intercept script will be
            skipped. Default is False (0). Optional.
    """
    print(force)


def getAvailableLocales():
    """Returns a collection of strings representing the Locales added to
    the Translation Manager, such as 'en' for English.

    Returns:
        list[str]: A collection of strings representing the Locales
            added to the Translation Manager.
    """
    return ["en"]


def getAvailableTerms():
    """Returns a collection of available terms defined in the
    translation system.

    Returns:
         list[str]: A collection of all of the terms available from the
            Translation Manager.
    """
    return ["term1", "term2"]


def getClientId():
    """Returns a hex-string that represents a number unique to the
    running client's session.

    You are guaranteed that this number is unique between all running
    clients.

    Returns:
        str: A special code representing the client's session in a
            unique way.
    """
    return "92247003"


def getConnectTimeout():
    """Returns the connect timeout in milliseconds for all
    client-to-gateway communication.

    This is the maximum amount of time that communication operations to
    the Gateway will be given to connect. The default is 10,000ms (10
    seconds).

    Returns:
        int: The current connect timeout, in milliseconds. Default is
            10,000 (ten seconds).
    """
    return 10000


def getConnectionMode():
    """Retrieves this client session's current connection mode.

    3 is read/write, 2 is read-only, and 1 is disconnected.

    Returns:
        int: The current connection mode for the client.
    """
    return 3


def getEdition():
    """Returns the "edition" of the Vision client - "standard",
    "limited", or "panel".

    Returns:
        str: The edition of the Vision module that is running the
            client.
    """
    return "standard"


def getGatewayAddress():
    """Returns the address of the gateway that the client is currently
    communicating with.

    Returns:
        str: The address of the Gateway that the client is communicating
            with.
    """
    return "http://localhost:8088/"


def getGatewayStatus(
    gatewayAddress, connectTimeoutMillis=None, socketTimeoutMillis=None
):
    """Returns a string that indicates the status of the Gateway.

    A status of RUNNING means that the Gateway is fully functional.
    Thrown exceptions return "ERROR" with the error message appended to
    the string.

    Args:
        gatewayAddress (str): The gateway address to ping, in the form
            of ADDR:PORT/main.
        connectTimeoutMillis (int): The maximum time in milliseconds to
            attempt to initially contact a Gateway. Optional.
        socketTimeoutMillis (int): The maximum time in milliseconds to
            wait for a response from a Gateway after initial connection
            has been established. Optional.

    Returns:
        str: A string that indicates the status of the Gateway. A status
            of RUNNING means that the Gateway is fully functional.
    """
    print(gatewayAddress, connectTimeoutMillis, socketTimeoutMillis)
    return "RUNNING"


def getGlobals():
    """This method returns a dictionary that provides access to the
    legacy global namespace.

    As of version 7.7.0, most new scripts use the modern style of
    scoping, which makes the 'global' keyword act very differently. Most
    importantly, the modern scoping rules mean that variables declared
    as 'global' are only global within that one module. The
    system.util.getGlobals() method can be used to interact with older
    scripts that used the old meaning of the 'global' keyword.

    Returns:
        dict: The global namespace, as a dictionary.
    """
    return {}


def getInactivitySeconds():
    """Returns the number of seconds since any keyboard or mouse
    activity.

    Returns:
        long: The number of seconds the mouse and keyboard have been
            inactive for this client.
    """
    long_instance = int if sys.version_info[0] == 3 else long
    return long_instance(0)


def getLocale():
    """Returns the current string representing the user's Locale, such
    as 'en' for English.

    Returns:
        str: The current Locale.
    """
    return "en"


def getLogger(name):
    """Returns a Logger object that can be used to log messages to the
    console.

    Args:
        name (str): The name of a logger to create.

    Returns:
        LoggerEx: A new Logger object used to log informational and
            error messages.
    """
    print(name)
    return LoggerEx()


def getProjectName():
    """Returns the name of the project that is currently being run.

    Returns:
        str: The name of the currently running project.
    """
    return "MyProject"


def getProperty(propertyName):
    r"""Retrieves the value of a named system property.

    Some of the available properties are:

        file.separator. The system file separator character. (for
            example, "/" (unix) or "\" (windows))
        line.separator. The system line separator string. (for example,
            "\r\n" (carriage return, newline))
        os.arch. Operating system architecture. (for example, "x86")
        os.name. Operating system name. (for example, "Windows XP")
        os.version. Operating system version. (for example, "5.1")
        user.home. User's home directory.
        user.name. User's account name.

    Args:
        propertyName (str): The name of the system property to get.

    Returns:
        str: The value for the named property.
    """
    # Initialize variables.
    ret = None

    if propertyName == "file.separator":
        ret = os.sep
    elif propertyName == "line.separator":
        ret = os.linesep
    elif propertyName == "os.arch":
        ret = platform.machine()
    elif propertyName == "os.name":
        ret = platform.system()
    elif propertyName == "os.version":
        ret = platform.release()
    elif propertyName == "user.home":
        ret = os.path.expanduser("~")
    elif propertyName == "user.name":
        ret = getpass.getuser()

    return ret


def getReadTimeout():
    """Returns the read timeout in milliseconds for all
    client-to-gateway communication.

    This is the maximum amount of time allowed for a communication
    operation to complete. The default is 60,000 ms (1 minute).

    Returns:
         int: The current read timeout, in milliseconds. Default is
            60,000 ms (one minute).
    """
    return 60000


def getSessionInfo(usernameFilter=None, projectFilter=None):
    """Returns a PyDataSet holding information about all of the sessions
    (logged-in users) on the Gateway.

    Optional regular-expression based filters can be provided to filter
    the username or the username and the project returned.

    Args:
        usernameFilter (str): A regular-expression based filter string
            to restrict the list by username. Optional.
        projectFilter (str): A regular-expression based filter string to
            restrict the list by project. Optional.

    Returns:
        PyDataSet: A dataset representing the Gateway's current
            sessions.
    """
    print(usernameFilter, projectFilter)
    return DatasetUtilities.PyDataSet()


def getSystemFlags():
    """Returns an integer that represents a bit field containing
    information about the currently running system.

    Each bit corresponds to a specific flag as defined in the bitmask
    below. The integer return will be a total of all of the bits that
    are currently active.

    Returns:
        int: A total of all the bits that are currently active. A full
            screen client launched from the gateway webpage with no SSL
            will have a value of 44 (Fullscreen flag + Webstart Flag +
            Client Flag).
    """
    return 1


def getVersion():
    """Returns the Ignition version number that is currently being run.

    Returns:
        Version: The currently running Ignition version number. as a
            Version object.
    """
    major, minor, rev = [int(i) for i in version.__version__.split(".")]
    build = int(version.__build__)
    return Version(major=major, minor=minor, rev=rev, build=build)


def invokeAsynchronous(function, args=None, kwargs=None, description=None):
    """Invokes (calls) the given Python function on a different thread.

    This means that calls to invokeAsynchronous will return immediately,
    and then the given function will start executing asynchronously on a
    different thread. This is useful for long-running data intensive
    functions, where running them synchronously (in the GUI thread)
    would make the GUI non-responsive for an unacceptable amount of
    time.

    Args:
        function (object): A Python function object that will get
            invoked with no arguments in a separate thread.
        args: A list or tuple of Python objects that will be provided to
            the called function as arguments. Equivalent to the *
            operator. Optional.
        kwargs: A dictionary of keyword argument names to Python object
            values that will be provided to the called function as
            keyword arguments. Equivalent to the ** operator. Optional.
        description (str): A description to use for the asynchronous
            thread. Will be displayed on the current scope's diagnostic
            view for scripts. For Vision and the Designer, this would be
            the "Scripts" tab of the Diagnostics popup. For Perspective
            and the Gateway scope, this would be the Gateway's Running
            Scripts status page. Optional.

    Returns:
        Thread: The executing thread.
    """
    print(function, args, kwargs, description)
    return Thread()


def invokeLater(function, delay=0):
    """Invokes (calls) the given Python function object after all of the
    currently processing and pending events are done being processed, or
    after a specified delay.

    The function will be executed on the GUI, or event dispatch, thread.
    This is useful for events like propertyChange events, where the
    script is called before any bindings are evaluated.

    If you specify an optional time argument (number of milliseconds),
    the function will be invoked after all currently processing and
    pending events are processed plus the duration of that time.

    Args:
        function (object): A Python function object that will be invoked
            later, on the GUI, or event-dispatch, thread with no
            arguments.
        delay (int): A delay, in milliseconds, to wait before the
            function is invoked. The default is 0, which means it will
            be invoked after all currently pending events are processed.
            Optional.
    """
    print(function, delay)


def jsonDecode(jsonString):
    """Takes a json String and converts it into a Python object such as
    a list or a dict.

    If the input is not valid json, a string is returned.

    Args:
        jsonString (str): The JSON string to decode into a Python
            object.

    Returns:
        dict: The decoded Python object.
    """
    return json.loads(jsonString)


def jsonEncode(pyObj, indentFactor=4):
    """Takes a Python object such as a list or dict and converts into a
    json string.

    Args:
        pyObj (object): The Python object to encode into JSON such as a
            Python list or dictionary.
        indentFactor (int): The number of spaces to add to each level of
            indentation for prettyprinting. Optional.

    Returns:
        str: The encoded JSON string.
    """
    return json.dumps(pyObj, indent=indentFactor)


def modifyTranslation(term, translation, locale="en"):
    """This function allows you to add or modify a global translation.

    Args:
        term (str): The key term to translate.
        translation (str): The translated value to store.
        locale (str): If specified, the locale code (such as "es")
            identifying the language of the translation. Otherwise, the
            currently set language is used. Optional.
    """
    print(term, translation, locale)


def playSoundClip(wav, volume=1.0, wait=False):
    """Plays a sound clip from a wav file to the system's default audio
    device.

    The wav file can be specified as a filepath, a URL, or directly as a
    raw byte[].

    Args:
        wav (object): A byte list of a wav file or filepath or URL that
            represents a wav file.
        volume (float): The clip's volume, represented as a floating
            point number between 0.0 and 1.0. Optional.
        wait (bool): A boolean flag indicating whether or not the call
            to playSoundClip should wait for the clip to finish before
            it returns. Optional.
    """
    print(wav, volume, wait)


def queryAuditLog(
    auditProfileName,
    startDate=None,
    endDate=None,
    actorFilter=None,
    actionFilter=None,
    targetFilter=None,
    valueFilter=None,
    systemFilter=None,
    contextFilter=None,
):
    """Queries an audit profile for audit history.

    Returns the results as a dataset.

    Args:
        auditProfileName (str): The name of the audit profile to pull
            the history from.
        startDate (Date): The earliest audit event to return. If
            omitted, the current time - 8 hours will be used. Optional.
        endDate (Date): The latest audit event to return. If
            omitted, the current time will be used. Optional.
        actorFilter (str): A filter string used to restrict the results
            by actor. Optional.
        actionFilter (str): A filter string used to restrict the results
            by action. Optional.
        targetFilter (str): A filter string used to restrict the results
            by target. Optional.
        valueFilter (str): A filter string used to restrict the results
            by value. Optional.
        systemFilter (str): A filter string used to restrict the results
            by system. Optional.
        contextFilter (int): A bitmask used to restrict the results by
            context. 0x01 = Gateway, 0x02 = Designer, 0x04 = Client.
            Optional.

    Returns:
        BasicDataset: A dataset with the audit events from the specified
            profile that match the filter arguments.
    """
    endDate = system.date.now() if endDate is None else endDate
    startDate = (
        system.date.addHours(endDate, -8) if startDate is None else startDate
    )
    print(
        auditProfileName,
        startDate,
        endDate,
        actorFilter,
        actionFilter,
        targetFilter,
        valueFilter,
        systemFilter,
        contextFilter,
    )
    return BasicDataset()


def retarget(project, addresses=None, params=None, windows=None):
    """This function allows you to programmatically 'retarget' the
    Client to a different project and/or different Gateway.

    You can have it switch to another project on the same Gateway, or
    another gateway entirely, even across a WAN. This feature makes the
    vision of a seamless, enterprise-wide SCADA application a reality.

    The retarget feature will attempt to transfer the current user
    credentials over to the new project / Gateway. If the credentials
    fail on that project, the user will be prompted for a valid username
    and password. Once valid authentication has been achieved, the
    currently running project is shut down, and the new project is
    loaded.

    You can pass any information to the other project through the
    parameters dictionary. All entries in this dictionary will be set in
    the global scripting namespace in the other project. Even if you
    don't specify any parameters, the system will set the variable
    _RETARGET_FROM_PROJECT to the name of the current project and
    _RETARGET_FROM_GATEWAY to the address of the current Gateway.

    Args:
        project (str): The name of the project to retarget to.
        addresses (object): The address of the Gateway that the project
            resides on. If omitted, the current Gateway will be used.
            Format is: host:port. Optional.
            As of 8.0.8 this can be a list of strings. When using a
            list, the function will try each address in order, waiting
            for the timeout period between each address attempt.
        params (dict): A dictionary of parameters that will be passed to
            the new project. They will be set as global variables in the
            new project's Python scripting environment. Optional.
        windows (list[str]): A list of window paths to use as the
            startup windows. If omitted, the project's normal startup
            windows will be opened. If specified, the project's normal
            startup windows will be ignored, and this list will be used
            instead. Optional.
    """
    print(project, addresses, params, windows)


def sendMessage(
    project,
    messageHandler,
    payload=None,
    scope=None,
    clientSessionId=None,
    user=None,
    hasRole=None,
    hostName=None,
    remoteServers=None,
):
    """This function sends a message to clients running under the
    Gateway, or to a project within the Gateway itself.

    To handle received messages, you must set up event script message
    handlers within a project. These message handlers run Jython code
    when a message is received. You can add message handlers under the
    "Message" section of the client/Gateway event script configuration
    dialogs.

    Args:
        project (str): The name of the project containing the message
            handler.
        messageHandler (str): The name of the message handler that will
            fire upon receiving a message.
        payload (dict): A PyDictionary which will get passed to the
            message handler. Use "payload" in the message handler to
            access dictionary variables. Optional.
        scope (str): Limits the scope of the message delivery to "C"
            (clients), "G" (Gateway), or "CG" for clients and the
            Gateway. Defaults to "C" if the user name, role or host name
            parameters are set, and to "CG" if none of these parameters
            are set. Optional.
        clientSessionId (str): Limits the message delivery to a client
            with the specified session ID. Optional.
        user (str): Limits the message delivery to clients where the
            specified user has logged in. Optional.
        hasRole (str): Limits the message delivery to any client where
            the logged in user has the specified user role. Optional.
        hostName (str): Limits the message delivery to the client that
            has the specified network host name. Optional.
        remoteServers (list[str]): A list of Strings representing
            Gateway Server names. The message will be delivered to each
            server in the list. Upon delivery, the message is
            distributed to the local Gateway and clients as per the
            other parameters. Optional.

    Returns:
        list[str]: A List of Strings containing information about each
            system that was selected for delivery, where each List item
            is comma-delimited.
    """
    print(
        project,
        messageHandler,
        payload,
        scope,
        clientSessionId,
        user,
        hasRole,
        hostName,
        remoteServers,
    )


def sendRequest(
    project,
    messageHandler,
    payload=None,
    hostName=None,
    remoteServer=None,
    timeoutSec=None,
):
    """This function sends a message to the Gateway, working in a
    similar manner to the sendMessage function, except sendRequest
    expects a response to the message.

    To handle received messages, you must set up Gateway Event Script
    message handlers within a project. These message handlers run Jython
    code when a message is received. You can then place a return at the
    end of the code to return something to where the sendRequest was
    originally called from. You can add message handlers under the
    "Message" section of the Gateway Event Script configuration dialog.

    Args:
        project (str): The name of the project containing the message
            handler.
        messageHandler (str): The name of the message handler that will
            fire upon receiving a message.
        payload (dict): A PyDictionary which will get passed to the
            message handler. Use "payload" in the message handler to
            access dictionary variables. Optional.
        hostName (str): Limits the message delivery to the client that
            has the specified network host name. Optional.
        remoteServer (str): A string representing a target Gateway
            Server name. The message will be delivered to the remote
            Gateway over the Gateway Network. Upon delivery, the message
            is distributed to the local Gateway and clients as per the
            other parameters. Optional.
        timeoutSec (str): The number of seconds before the sendRequest
            call times out. Optional.

    Returns:
        object: The return from the message handler.
    """
    print(
        project,
        messageHandler,
        payload,
        hostName,
        remoteServer,
        timeoutSec,
    )


def sendRequestAsync(
    project,
    messageHandler,
    payload=None,
    hostName=None,
    remoteServer=None,
    timeoutSec=None,
    onSuccess=None,
    onError=None,
):
    """This function sends a message to the Gateway and expects a
    response.

    Works in a similar manner to the sendRequest function, except
    sendRequestAsync will send the request and then immediately return a
    handle for it.

    Args:
        project (str): The name of the project containing the message
            handler.
        messageHandler (str): The name of the message handler that will
            fire upon receiving a message.
        payload (dict): A PyDictionary which will get passed to the
            message handler. Use "payload" in the message handler to
            access dictionary variables. Optional.
        hostName (str): Limits the message delivery to the client that
            has the specified network host name. Optional.
        remoteServer (str): A string representing the target Gateway
            Server name. The message will be delivered to the remote
            Gateway over the Gateway Network. Upon delivery, the message
            is distributed to the local Gateway and clients as per the
            other parameters. Optional.
        timeoutSec (int): The number of seconds before the sendRequest
            call times out. Optional.
        onSuccess (object): Should take one argument, which will be the
            result from the message handler. Callback functions will be
            executed on the GUI thread, similar to
            system.util.invokeLater. Optional.
        onError (object): Should take one argument, which will be the
            exception encountered. Callback functions will be executed
            on the GUI thread, similar to system.util.invokeLater.
            Optional.

    Returns:
        Request: The Request object that can be used while waiting for
            the message handler callback.
    """
    print(
        project,
        messageHandler,
        payload,
        hostName,
        remoteServer,
        timeoutSec,
        onSuccess,
        onError,
    )
    return SystemUtilities.RequestImpl(1000)


def setConnectTimeout(connectTimeout):
    """Sets the connect timeout for client-to-gateway communication.

    Specified in milliseconds.

    Args:
        connectTimeout (int): The new connect timeout, specified in
            milliseconds.
    """
    print(connectTimeout)


def setConnectionMode(mode):
    """Sets the connection mode for the client session.

    Normally a client runs in mode 3, which is read-write. You may wish
    to change this to mode 2, which is read-only, which will only allow
    reading and subscribing to tags, and running SELECT queries. Tag
    writes and INSERT / UPDATE / DELETE queries will not function. You
    can also set the connection mode to mode 1, which is disconnected,
    all tag and query features will not work.

    Args:
        mode (int): The new connection mode. 1 = Disconnected,
            2 = Read-only, 3 = Read/Write.
    """
    print(mode)


def setLocale(locale):
    """Sets the user's current Locale.

    Any valid Java locale code (case-insensitive) can be used as a
    parameter, including ones that have not yet been added to the
    Translation Manager. An invalid locale code will cause an Illegal
    Argument Exception.

    Args:
        locale (str): A locale code, such as 'en_US' for US English.
    """
    print(locale)


def setLoggingLevel(loggerName, loggerLevel):
    """Sets the logging level on the given logger.

    This can be a logger you create, or a logger already defined in the
    system.

    Args:
        loggerName (str): The unique name of the logger to change the
            logging level on, for example "Tags.Client".
        loggerLevel (str): The level you want to change to logger to:
            "trace", "debug", "info", "warn" or "error".
    """
    print(loggerName, loggerLevel)


def setReadTimeout(readTimeout):
    """Sets the read timeout for client-to-gateway communication.

    Specified in milliseconds.

    Args:
        readTimeout (int): The new read timeout, specified in
            milliseconds.
    """
    print(readTimeout)


def threadDump():
    """Creates a thread dump of the current running JVM.

    Returns:
        str: The dump of the current running JVM.
    """
    return "Ignition version: {}...".format(getVersion())


def translate(term, locale=None, strict=False):
    """This function allows you to retrieve the global translation of a
    term from the translation database using the current locale.

    Args:
        term (str): The term to look up.
        locale (str): Which locale to translate against. Useful when
            there are multiple locales defined for a single term. If
            omitted, the function attempts to use the current locale (as
            defined by the client, session, or Designer). Optional.
        strict (bool): If False, the function will return the passed
            term (param 1) if it could not find a defined translation
            for the locale: meaning, if you pass a term that hasn't been
            configured, the function will just send the term back to
            you. If True, then the function will return a None when it
            fails to find a defined translation. Default is False.
            Optional.

    Returns:
        str: The translated term.
    """
    print(term, locale, strict)
    return term



# coding=utf-8

"""Package information."""

__title__ = "ignition-api"
__description__ = "Ignition Scripting API."
__url__ = "https://github.com/thecesrom/Ignition"
__version__ = "8.1.10"
__cycle__ = ".post7"
__build__ = "2021090812"
__author__ = "César Román"
__author_email__ = "cesar@thecesrom.dev"
__license__ = "MIT"
__copyright__ = "Copyright (C) 2018-2021 César Román"



"""Alarm Functions.

The following functions give you access to view and interact with the
Alarm system in Ignition.
"""

from __future__ import print_function

__all__ = [
    "acknowledge",
    "cancel",
    "createRoster",
    "getRosters",
    "getShelvedPaths",
    "listPipelines",
    "queryJournal",
    "queryStatus",
    "shelve",
    "unshelve",
]

import system.date
from com.inductiveautomation.ignition.common.alarming.evaluation import (
    ShelvedPath,
)
from com.inductiveautomation.ignition.common.alarming.query import (
    AlarmQueryResultImpl,
)
from java.util import Date


def acknowledge(alarmIds, notes=None, username=None):
    """Acknowledges any number of alarms, specified by their event ids.

    The event id is generated for an alarm when it becomes active, and
    is used to identify a particular event from other events for the
    same source. The alarms will be acknowledged by the logged in user
    making the call. Additionally, acknowledgement notes may be included
    and will be stored along with the acknowledgement.

    Args:
        alarmIds (list[str]): List of alarm event ids (uuids) to
            acknowledge.
        notes (str): Notes that will be stored on the acknowledged alarm
            events. Optional.
        username (str): The user that acknowledged the alarm. NOTE that
            this parameter is only used when called from a gateway
            scoped script. This parameter should be omitted from any
            client-based scripts. Optional.
    """
    print(alarmIds, notes, username)


def cancel(alarmIds):
    """Cancels any number of alarms, specified by their event ids.

    The event id is generated for an alarm when it becomes active, and
    is used to identify a particular event from other events for the
    same source. The alarm will still be active, but will drop out of
    alarm pipelines.

    Args:
        alarmIds (list[str]): List of alarm event ids (uuids) to cancel.
    """
    print(alarmIds)


def createRoster(name, description=""):
    """This function creates a new roster.

    Users may be added to the roster through the Gateway or the Roster
    Management component.

    Args:
        name (str): The name for the new roster.
        description (str): A description for the new roster. Required,
            but can be blank. Optional.
    """
    print(name, description)


def getRosters():
    """This function returns a mapping of roster names to a list of
    usernames contained in the roster.

    Returns:
         dict: A dictionary that maps roster names to a List of
            usernames in the roster. The List of usernames may be empty
            if no users have been added to the roster.
    """
    return {}


def getShelvedPaths():
    """A list of ShelvedPath objects.

    ShelvedPath objects can be examined with getExpiration, getHitCount,
    getPath, getShelveTime, getUser, and isExpired.

    Returns:
        list[ShelvedPath]: A list of ShelvedPath objects. ShelvedPath
            objects can be examined with getExpiration, getHitCount,
            getPath, getShelveTime, getUser, and isExpired.
    """
    return [ShelvedPath()]


def listPipelines(projectName="alarm-pipelines"):
    """Will return a list of the available Alarm Notification Pipelines.

    Args:
        projectName (str): The project to check alarm pipelines for. If
            omitted, will look for a project named "alarm-pipelines".
            Optional.

    Returns:
        list[str]: A list of pipeline names. The list may be empty if no
            pipelines exist. Unsaved name changes will not be reflected
            in the list.
    """
    print(projectName)
    return []


def queryJournal(
    startDate=None,
    endDate=None,
    journalName=None,
    priority=None,
    state=None,
    path=None,
    source=None,
    displaypath=None,
    all_properties=None,
    any_properties=None,
    defined=None,
    includeData=None,
    includeSystem=None,
    isSystem=None,
):
    """Queries the specified journal for historical alarm events.

    The result is a list of alarm events, which can be queried for
    individual properties. The result object also has a getDataset()
    function that can be used to convert the query results into a normal
    dataset, with the columns: EventId, Source, DisplayPath, EventTime,
    EventState, Priority, IsSystemEvent.

    Args:
        startDate (Date): The start of the time range to query.
            Defaults to 8 hours previous to now if omitted. Time range
            is inclusive.
        endDate (Date): The end of the time range to query. Defaults
            to "now" if omitted.
        journalName (str): The journal name to query.
        priority (list[str]): A list of possible priorities to match.
            Priorities can be specified by name or number, with the
            values: Diagnostic(0), Low(1), Medium(2), High(3),
            Critical(4).
        state (list[str]): A list of the event state types to match.
            Valid values are "ClearUnacked", "ClearAcked",
            "ActiveUnacked", and "ActiveAcked".
        path (list[str]): A list of possible source paths to search at.
            The wildcard "*" may be used.
        source (list[str]): A list of possible source paths to search
            at. The wildcard "*" may be used.
        displaypath (list[str]): A list of display paths to search at.
            Display paths are separated by "/", and if a path ends in
            "/*", everything below that path will be searched as well.
        all_properties (list[object]): A set of property conditions, all
            of which must be met for the condition to pass. This
            parameter is a list of tuples, in the form ("propName",
            "condition", value). Valid condition values: "=", "!=", "<",
            "<=", ">", ">=". Only the first two conditions may be used
            for string values.
        any_properties (list[object]): A set of property conditions, any
            of which will cause the overall the condition to pass. This
            parameter is a list of tuples, in the form ("propName",
            "condition", value). Valid condition values: "=", "!=", "<",
            "<=", ">", ">=". Only the first two conditions may be used
            for string values.
        defined (list[str]): A list of string property names, all of
            which must be present on an event for it to pass.
        includeData (bool): Whether or not event data should be included
            in the return. If this parameter is False, and if there are
            no conditions specified on associated data, the properties
            table will not be queried.
        includeSystem (bool): Specifies whether system events are
            included in the return.
        isSystem (bool): Specifies whether the returned event must or
            must not be a system event.

    Returns:
        AlarmQueryResultImpl: The AlarmQueryResult object is
            functionally a list of AlarmEvent objects. The
            AlarmQueryResult object has a built-in getDataset() function
            that will return a Standard Dataset containing the Event Id
            (UUID of the alarm), Source Path, Display Path, Event Time,
            State (as an integer), and Priority (as an integer).
    """
    endDate = system.date.now() if endDate is None else endDate
    startDate = (
        system.date.addHours(endDate, -8) if startDate is None else startDate
    )
    print(
        startDate,
        endDate,
        journalName,
        priority,
        state,
        path,
        source,
        displaypath,
        all_properties,
        any_properties,
        defined,
        includeData,
        includeSystem,
        isSystem,
    )
    return AlarmQueryResultImpl()


def queryStatus(
    priority=None,
    state=None,
    path=None,
    source=None,
    displaypath=None,
    all_properties=None,
    any_properties=None,
    defined=None,
    includeShelved=False,
):
    """Queries the current state of alarms.

    The result is a list of alarm events, which can be queried for
    individual properties. The result object also has a getDataset()
    function that can be used to convert the query results into a normal
    dataset, with the columns: EventId, Source, DisplayPath, EventTime,
    State, Priority.

    Args:
        priority (list[str]): A list of possible priorities to match.
            Priorities can be specified by name or number, with the
            values: Diagnostic(0), Low(1), Medium(2), High(3),
            Critical(4). Optional.
        state (list[str]): A list of states to allow. Valid values:
            "ClearUnacked", "ClearAcked", "ActiveUnacked",
            "ActiveAcked". Optional.
        path (list[str]): A list of possible source paths to search at.
            The wildcard "*" may be used. Works the same as the source
            argument, and either can be used. Optional.
        source (list[str]): A list of possible source paths to search
            at. The wildcard "*" may be used. Works the same as the path
            argument, and either can be used. Optional.
        displaypath (list[str]): A list of display paths to search at.
            Display paths are separated by "/", and if a path ends in
            "/*", everything below that path will be searched as well.
            Optional.
        all_properties (list[object]): A set of property conditions, all
            of which must be met for the condition to pass. This
            parameter is a list of tuples, in the form ("propName",
            "condition", value). Valid condition values: "=", "!=", "<",
            "<=", ">", ">=". Only the first two conditions may be used
            for string values. Optional.
        any_properties (list[object]): A set of property conditions, any
            of which will cause the overall the condition to pass. This
            parameter is a list of tuples, in the form ("propName",
            "condition", value). Valid condition values: "=", "!=", "<",
            "<=", ">", ">=". Only the first two conditions may be used
            for string values. Optional.
        defined (list[str]): A list of string property names, all of
            which must be present on an event for it to pass. Optional.
        includeShelved (bool): A flag indicating whether shelved events
            should be included in the results. Defaults to "False".
            Optional.

    Returns:
        AlarmQueryResultImpl: The AlarmQueryResult object is
            functionally a list of AlarmEvent objects. The
            AlarmQueryResult object has a built-in getDataset() function
            that will return a Standard Dataset containing the Event Id
            (UUID of the alarm), Source Path, Display Path, Event Time,
            State (as an integer), and Priority (as an integer).
    """
    print(
        priority,
        state,
        path,
        source,
        displaypath,
        all_properties,
        any_properties,
        defined,
        includeShelved,
    )
    return AlarmQueryResultImpl()


def shelve(path, timeoutSeconds=None, timeoutMinutes=None):
    """This function shelves the specified alarms for the specified
    amount of time.

    The paths may be either source paths, or display paths. The time can
    be specified in minutes (timeoutMinutes) or seconds
    (timeoutSeconds). If an alarm is already shelved, this will
    overwrite the remaining time. To unshelve alarms, this function may
    be used with a time of "0".

    Args:
        path (list[str]): A list of possible source paths to search at.
            If a path ends in "/*", the results will include anything
            below that path.
        timeoutSeconds (int): The amount of time to shelve the matching
            alarms for, specified in seconds. 0 indicates that matching
            alarm events should now be allowed to pass. Optional.
        timeoutMinutes (int): The amount of time to shelve the matching
            alarms for, specified in minutes. 0 indicates that matching
            alarm events should now be allowed to pass. Optional.
    """
    print(path, timeoutSeconds, timeoutMinutes)


def unshelve(path):
    """Unshelves alarms in accordance with the path parameter.

    Args:
        path (list[str]): A list of possible source paths to search at.
            If a path ends in "/*", the results will include anything
            below that path.
    """
    print(path)



"""Net Functions.

The following functions give you access to interact with http services.
"""

from __future__ import print_function

__all__ = [
    "getExternalIpAddress",
    "getHostName",
    "getIpAddress",
    "getRemoteServers",
    "httpClient",
    "httpDelete",
    "httpGet",
    "httpPost",
    "httpPut",
    "openURL",
    "sendEmail",
]

import socket

from com.inductiveautomation.ignition.common.script.builtin.http import (
    JythonHttpClient,
)


def getExternalIpAddress():
    """Returns the client's IP address, as it is detected by the
    Gateway.

    This means that this call will communicate with the Gateway, and the
    Gateway will tell the client what IP address its incoming traffic is
    coming from. If you have a client behind a NAT router, then this
    address will be the WAN address of the router instead of the LAN
    address of the client, which is what you'd get with
    system.net.getIpAddress.

    Returns:
        str: A text representation of the client's IP address, as
            detected by the Gateway.
    """
    return "52.52.32.221"


def getHostName():
    """Returns the host name of the computer that the script was ran on.

    When run in the Gateway scope, returns the Gateway hostname. When
    run in the Client scope, returns the Client hostname. On Windows,
    this is typically the "computer name". For example, might return
    EAST_WING_WORKSTATION or bobs-laptop.

    Returns:
        str: The hostname of the local machine.
    """
    return socket.gethostname()


def getIpAddress():
    """Returns the IP address of the computer the client is running on,
    as it appears to the client.

    See also: system.net.getExternalIpAddress().

    Returns:
        str: Returns the IP address of the local machine, as it sees it.
    """
    return "127.0.0.1"


def getRemoteServers(runningOnly=True):
    """This function returns a List of Gateway Network servers that are
    visible from the local Gateway.

    Args:
        runningOnly (bool): If set to True, only servers on the Gateway
            Network that are running will be returned. Servers that have
            lost contact with the Gateway Network will be filtered out.
            Optional.

    Returns:
        list[str]: A List of Strings representing Gateway Network server
            ids.
    """
    print(runningOnly)
    return []


def httpClient(
    timeout=60000,
    bypass_cert_validation=True,
    username=None,
    password=None,
    proxy=None,
    cookie_policy="ACCEPT_ORIGINAL_SERVER",
    redirect_policy="NORMAL",
    customizer=None,
):
    """Provides a general use object that can be used to send and
    receive HTTP requests.

    The object created by this function is a wrapper around Java's
    HttpClient class. Usage requires creating a JythonHttpClient object
    with a call to system.net.httpClient, then calling a method (such as
    get(), post()) on the JythonHttpClient to actually issue a request.

    Args:
        timeout (int): A value, in milliseconds, to set the client's
            connect timeout setting to. Defaults to 60000. Optional.
        bypass_cert_validation (bool): A boolean indicating whether the
            client should attempt to validate the certificates of remote
            servers, if connecting via HTTPS/SSL. Defaults to True.
            Optional.
        username (str): A string indicating the username to use for
            authentication if the remote server requests authentication;
            specifically, by responding with a WWW-Authenticate or
            Proxy-Authenticate header. Only supports Basic
            authentication. If username is specified but not password,
            an empty string will be used for the password in the Basic
            Authentication response. Defaults to None. Optional.
        password (str): A string indicating the password to use for
            authentication. Defaults to None. Optional.
        proxy (str): The address of a proxy server, which will be used
            for HTTP and HTTPS traffic. If a port is not specified as
            part of that address, it will be assumed from the protocol
            in the URL, i.e. 80/443. Defaults to None. Optional.
        cookie_policy (str): A string representing this client's cookie
            policy. Accepts values "ACCEPT_ALL", "ACCEPT_NONE", and
            "ACCEPT_ORIGINAL_SERVER". Defaults to
            "ACCEPT_ORIGINAL_SERVER". Optional.
        redirect_policy (str): A string representing this client's
            redirect policy. Acceptable values are listed below.
            Defaults to "Normal". Optional.
        customizer (object): A reference to a function. This function
            will be called with one argument (an instance of
            HttpClient.Builder). The function should operate on that
            builder instance, which allows for customization of the
            created HTTP client. Defaults to None. Optional.

    Returns:
        JythonHttpClient: An object wrapped around an instance of Java's
            HttpClient class. The httpClient object has methods that can
            be called to execute HTTP requests against a server.
    """
    print(
        timeout,
        bypass_cert_validation,
        username,
        password,
        proxy,
        cookie_policy,
        redirect_policy,
        customizer,
    )
    return JythonHttpClient()


def httpDelete(
    url,
    contentType=None,
    connectTimeout=10000,
    readTimeout=60000,
    username=None,
    password=None,
    headerValues=None,
    bypassCertValidation=True,
):
    """Performs an HTTP DELETE to the given URL.

    Args:
        url (str): The URL to send the request to.
        contentType (str): The MIME type used in the HTTP 'Content-type'
            header. Optional.
        connectTimeout (int): The timeout for connecting to the URL in
            milliseconds. Default is 10,000. Optional.
        readTimeout (int): The read timeout for the operation in
            milliseconds. Default is 60,000. Optional.
        username (str): If specified, the call will attempt to
            authenticate with basic HTTP authentication. Optional.
        password (str): The password used for basic HTTP authentication,
            if the username parameter is also present. Optional.
        headerValues (dict): A dictionary of name/value pairs that will
            be set in the HTTP header. Optional.
        bypassCertValidation (bool): If the target address in an HTTPS
            address, and this parameter is TRUE, the system will bypass
            all SSL certificate validation. This is not recommended,
            though is sometimes necessary for self-signed certificates.
            Optional.

    Returns:
        object: The content returned for the DELETE operation.
    """
    print(
        url,
        contentType,
        connectTimeout,
        readTimeout,
        username,
        password,
        headerValues,
        bypassCertValidation,
    )
    return object


def httpGet(
    url,
    connectTimeout=10000,
    readTimeout=60000,
    username=None,
    password=None,
    headerValues=None,
    bypassCertValidation=None,
    useCaches=True,
    throwOnError=True,
):
    """Retrieves the document at the given URL using the HTTP GET
    protocol.

    The document is returned as a string. For example, if you use the
    URL of a website, you'll get the same thing you'd get by going to
    that website in a browser and using the browser's "View Source"
    function.

    Args:
        url (str): The URL to retrieve.
        connectTimeout (int): The timeout for connecting to the URL. In
            milliseconds. Default is 10,000. Optional.
        readTimeout (int): The read timeout for the get operation. In
            milliseconds. Default is 60,000. Optional.
        username (str): If specified, the call will attempt to
            authenticate with basic HTTP authentication. Optional.
        password (str): The password used for basic HTTP authentication,
            if the username parameter is also present. Optional.
        headerValues (dict): A dictionary of name/value pairs that will
            be set in the HTTP header. Optional.
        bypassCertValidation (bool): If the target address is an HTTPS
            address, and this parameter is True, the system will bypass
            all SSL certificate validation. This is not recommended,
            though is sometimes necessary for self-signed certificates.
            Optional.
        useCaches (bool): Will cache the information returned by the
            httpGet call. If using this for something that constantly
            updates like an rss feed, it would be better to set this to
            False. Default is True. Optional.
        throwOnError (bool): Set to False if you wish to get the error
            body rather than a Python exception if the GET request
            returns an error code (non-200 responsive). Default is True.
            Optional.

    Returns:
        str: The content found at the given URL.
    """
    print(
        url,
        connectTimeout,
        readTimeout,
        username,
        password,
        headerValues,
        bypassCertValidation,
        useCaches,
        throwOnError,
    )
    return ""


def httpPost(url, *args):
    """Retrieves the document at the given URL using the HTTP POST
    protocol.

    If a parameter dictionary argument is specified, the entries in the
    dictionary will encoded in "application/x-www-form-urlencoded"
    format, and then posted. You can post arbitrary data as well, but
    you'll need to specify the MIME type. The document is then returned
    as a string.

    Args:
        url (str): The URL to post to.
        *args: Variable length argument list.

    Returns:
        str: The content returned for the POST operation.
    """
    print(url, args)
    return ""


def httpPut(url, *args, **kwargs):
    """Performs an HTTP PUT to the given URL.

    Encodes the given dictionary of parameters using
    "applications/x-www-form-urlencoded" format.

    Args:
        url (str): The URL to send the request to.
        *args: Variable length argument list.
        **kwargs: Arbitrary keyword arguments.

    Returns:
        str: The content returned by the PUT operation.
    """
    print(url, args, kwargs)
    return ""


def openURL(url, useApplet=False):
    """Opens the given URL or URI scheme outside of the currently
    running Client in whatever application the host operating system
    deems appropriate.

    Args:
        url (str): The URL to open in a web browser.
        useApplet (bool): If set to True (1), and the client is running
            as an Applet, then the browser instance that launched the
            applet will be used to open the URL. Optional.
    """
    print(url, useApplet)


def sendEmail(
    smtp,
    fromAddr,
    subject,
    body,
    html,
    to,
    attachmentNames=None,
    attachmentData=None,
    timeout=300000,
    username=None,
    password=None,
    priority="3",
    smtpProfile=None,
    cc=None,
    bcc=None,
    retries=0,
    replyTo=None,
):
    """Sends an email through the given SMTP server.

    Note that this email is relayed first through the Gateway - the
    client host machine doesn't need network access to the SMTP server.

    Args:
        smtp (str): The address of an SMTP server to send the email
            through, like "mail.example.com". A port can be specified,
            like "mail.example.com:25". SSL can also be forced, like
            "mail.example.com:25:tls".
        fromAddr (str): An email address to have the email come from.
        subject (str): The subject line for the email.
        body (str): The body text of the email.
        html (bool): A flag indicating whether or not to send the email
            as an HTML email. Will auto-detect if omitted.
        to (list[str]): A list of email addresses to send to.
        attachmentNames (list[str]): A list of attachment names.
            Attachment names must have the correct extension for the
            file type or an error will occur.
        attachmentData (list[object]): A list of attachment data, in
            binary format.
        timeout (int): A timeout for the email, specified in
            milliseconds. Defaults to 5 minutes (60,000*5). Optional.
        username (str): If specified, will be used to authenticate with
            the SMTP host. Optional.
        password (str): If specified, will be used to authenticate with
            the SMTP host. Optional.
        priority (str): Priority for the message, from "1" to "5", with
            "1" being highest priority. Defaults to "3" (normal)
            priority. Optional.
        smtpProfile (str): If specified, the named SMTP profile defined
            in the Gateway will be used. If this keyword is present, the
            smtp, username, and password keywords will be ignored.
            Optional.
        cc (list[str]): A list of email addresses to carbon copy. Only
            available if a smtpProfile is used. Optional.
        bcc (list[str]): A list of email addresses to blind carbon copy.
            Only available if a smtpProfile is used. Optional.
        retries (int): The number of additional times to retry sending
            on failure. Defaults to 0. Only available if a smtpProfile
            is used. Optional.
        replyTo (list[str]): An optional list of addresses to have the
            recipients reply to. If omitted, this defaults to the from
            address. Optional.
    """
    print(
        smtp,
        fromAddr,
        subject,
        body,
        html,
        to,
        attachmentNames,
        attachmentData,
        timeout,
        username,
        password,
        priority,
        smtpProfile,
        cc,
        bcc,
        retries,
        replyTo,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Ignition API."""

import os
from codecs import open

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, "src", "system", "__version__.py"), "r") as f:
    exec (f.read(), about)

with open("README.md", "r", "utf-8") as f:
    readme = f.read()

setup(
    name=about["__title__"],
    version="{}{}".format(about["__version__"], about["__cycle__"]),
    description=about["__description__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    url=about["__url__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    license=about["__license__"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Manufacturing",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing :: Mocking",
    ],
    keywords="hmi, ignition, inductive automation, scada",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=2.7.18",
)



"""Provides classes and interfaces that deal with editable and
noneditable text components. Examples of text components are text fields
and text areas, of which password fields and document editors are
special instantiations. Features that are supported by this package
include selection/highlighting, editing, style, and key mapping.
"""

from java.awt import Container


class JTextComponent(Container):
    """JTextComponent is the base class for swing text components."""

    _text = "Text"

    def getText(self):
        """Returns the text contained in this TextComponent."""
        return self._text

    def setText(self, t):
        """Sets the text of this TextComponent to the specified text.
        If the text is null or empty, has the effect of simply deleting
        the old text. When text has been inserted, the resulting caret
        location is determined by the implementation of the caret class.

        Args:
            t (str): The new text to be set.
        """
        self._text = t



from __future__ import print_function

__all__ = ["PyJsonObjectAdapter"]

from java.lang import Object


class PyJsonObjectAdapter(Object):
    def __init__(self, obj):
        print(self, obj)

    def __delitem__(self, key):
        pass

    def __findattr_ex__(self, name):
        pass

    def __finditem__(self, key):
        pass

    def __iter__(self):
        pass

    def __len__(self):
        pass

    def __setitem__(self, key, value):
        pass

    def clear(self):
        pass

    def get(self, key, default=None):
        pass

    def has_key(self, key):
        pass

    def items(self):
        pass

    def iteritems(self):
        pass

    def iterkeys(self):
        pass

    def itervalues(self):
        pass

    def keys(self):
        pass

    def pop(self, key):
        pass

    def popitem(self):
        pass

    def setdefault(self, key, default):
        pass

    def update(self, *args, **kwargs):
        pass

    def values(self):
        pass



"""OPC - UA Functions.

The following functions allow you to interact directly with an OPC-UA
server.
"""

from __future__ import print_function

__all__ = ["addConnection", "callMethod", "removeConnection"]


def addConnection(
    name,
    description,
    discoveryUrl,
    endpointUrl,
    securityPolicy,
    securityMode,
    settings,
):
    """Adds a new OPC UA connection.

    Args:
        name (str): Name to assign to the new connection.
        description (str): Description assigned to the new OPC UA
            connection.
        discoveryUrl (str): Endpoint URL to use for discovery
            services.
        endpointUrl (str): Endpoint URL to use for session services.
        securityPolicy (str): The name of the SecurityPolicy to use.
        securityMode (str): The name of the MessageSecurityMode to
            use.
        settings (dict): A dictionary of additional settings to apply
            to the connection.
    """
    print(
        name,
        description,
        discoveryUrl,
        endpointUrl,
        securityPolicy,
        securityMode,
        settings,
    )


def callMethod(connectionName, objectId, methodId, inputs):
    """Calls a method in an OPC UA server. To make the most of this
    function, you'll need to be familiar with methods in the OPC-UA
    server.

    Args:
        connectionName (str): The name of the OPC-UA connection to the
            server that the method resides in.
        objectId (str): The NodeId of the Object Node the Method is a
            member of.
        methodId (str): The NodeId of the Method Node to call.
        inputs (list[object]): A list of input values expected by the
            method.

    Returns:
        tuple: A tuple containing the following:
            0: Resulting StatusCode for the call
            1: A list of StatusCode objects corresponding to each
                input argument
            2: A list of output values

    """
    print(connectionName, objectId, methodId, inputs)


def removeConnection(name):
    """Removes an OPC UA Connection.

    Args:
        name (str): The name of the OPC UA connection to remove.

    Returns:
        bool: A boolean value representing whether the function was
            able to remove the connection. Returns True if the
            connection was successfully removed. Returns False if the
            connection was not removed.
    """
    print(name)
    return True



from __future__ import print_function

__all__ = ["BrowseTag", "OPCBrowseTag"]

from java.lang import Object


class BrowseTag(Object):
    def __init__(
        self,
        name=None,
        path=None,
        fullPath=None,
        type=None,
        valueSource=None,
        dataType=None,
    ):
        self.name = name
        self.path = path
        self.fullPath = fullPath
        self.type = type
        self.valueSource = valueSource
        self.dataType = dataType

    def getDataType(self):
        return self.dataType

    def getFullPath(self):
        return self.fullPath

    def getPath(self):
        return self.path

    def getTagType(self):
        return self.type

    def getValueSource(self):
        return self.valueSource

    def isDB(self):
        print(self)
        return True

    def isExpression(self):
        print(self)
        return True

    def isFolder(self):
        print(self)
        return True

    def isMemory(self):
        print(self)
        return True

    def isOPC(self):
        print(self)
        return True

    def isQuery(self):
        print(self)
        return True

    def isUDT(self):
        print(self)
        return True


class OPCBrowseTag(Object):
    def __init__(
        self,
        opcServer=None,
        type=None,
        displayName=None,
        displayPath=None,
        dataType=None,
        opcItemPath=None,
    ):
        self.opcServer = opcServer
        self.type = type
        self.displayName = displayName
        self.displayPath = displayPath
        self.dataType = dataType
        self.opcItemPath = opcItemPath

    def getDataType(self):
        return self.dataType

    def getDisplayName(self):
        return self.displayName

    def getDisplayPath(self):
        return self.displayPath

    def getOpcItemPath(self):
        return self.opcItemPath

    def getOpcServer(self):
        return self.opcServer

    def getType(self):
        return self.type



"""File Functions.

The following functions give you access to read and write to files.
"""

from __future__ import print_function

__all__ = [
    "fileExists",
    "getTempFile",
    "openFile",
    "openFiles",
    "readFileAsBytes",
    "readFileAsString",
    "saveFile",
    "writeFile",
]

import io
import os.path
import tempfile


def fileExists(filepath):
    """Checks to see if a file or folder at a given path exists.

    Args:
        filepath (str): The path of the file or folder to check.

    Returns:
        bool: True (1) if the file/folder exists, False (0) otherwise.
    """
    return os.path.isfile(filepath)


def getTempFile(extension):
    """Creates a new temp file on the host machine with a certain
    extension, returning the path to the file.

    The file is marked to be removed when the Java VM exits.

    Args:
        extension (str): An extension, like ".txt", to append to the end
            of the temporary file.

    Returns:
        str: The path to the newly created temp file.
    """
    suffix = ".{}".format(extension)
    with tempfile.NamedTemporaryFile(suffix=suffix) as temp:
        name = temp.name
    return name


def openFile(extension=None, defaultLocation=None):
    r"""Shows an "Open File" dialog box, prompting the user to choose a
    file to open.

    Returns the path to the file that the user chose, or None if the
    user canceled the dialog box. An extension can optionally be passed
    in that sets the filetype filter to that extension.

    Args:
        extension (str): A file extension, like "pdf", to try to open.
            Optional.
        defaultLocation (str): A folder location, like "C:\\MyFiles", to
            use as the starting location for the file chooser. Optional.

    Returns:
        str: The path to the selected file, or None if canceled.
    """
    print(extension, defaultLocation)


def openFiles(extension=None, defaultLocation=None):
    r"""Shows an "Open File" dialog box, prompting the user to choose a
    file or files to open.

    Returns the paths to the files that the user chooses, or None if the
    user canceled the dialog box. An extension can optionally be passed
    in that sets the filetype filter to that extension.

    Args:
        extension (str): A file extension, like "pdf", to try to open.
            Optional.
        defaultLocation (str): A folder location, like "C:\\MyFiles", to
            use as the starting location for the file chooser. Optional.

    Returns:
        list[str]: The paths to the selected files, or None if canceled.
    """
    print(extension, defaultLocation)


def readFileAsBytes(filepath):
    """Opens the file found at path filename, and reads the entire file.

    Returns the file as an array of bytes. Commonly this array of bytes
    is uploaded to a database table with a column of type BLOB (Binary
    Large OBject). This upload would be done through an INSERT or UPDATE
    SQL statement run through the system.db.runPrepUpdate function. You
    could also write the bytes to another file using the
    system.file.writeFile function, or send the bytes as an email
    attachment using system.net.sendEmail.

    Args:
        filepath (str): The path of the file to read.

    Returns:
        bytearray: The contents of the file as an array of bytes.
    """
    with open(filepath, "rb") as _file:
        return _file.read()


def readFileAsString(filepath, encoding="UTF-8"):
    """Opens the file found at path filename, and reads the entire file.

    Returns the file as a string. Common things to do with this string
    would be to load it into the text property of a component, upload it
    to a database table, or save it to another file using
    system.file.writeFile function.

    Args:
        filepath (str): The path of the file to read.
        encoding (str): The character encoding of the file to be read.
            Will throw an exception if the string does not represent a
            supported encoding. Common encodings are "UTF-8",
            "ISO-8859-1" and "US-ASCII". Optional.

    Returns:
        str: The contents of the file as a string.
    """
    with io.open(filepath, "r", encoding=encoding) as _file:
        return _file.read()


def saveFile(filename, extension=None, typeDesc=None):
    """Prompts the user to save a new file named filename.

    The optional extension and typeDesc arguments can be added to be
    used as a type filter. If the user accepts the save, the path to
    that file will be returned. If the user cancels the save, None will
    be returned.

    Args:
        filename (str): A file name to suggest to the user.
        extension (str): The appropriate file extension, like "jpeg",
            for the file. Optional.
        typeDesc (str): A description of the extension, like "JPEG
            Image". Optional.

    Returns:
        str: The path to the file that the user decided to save to, or
            None if they canceled.
    """
    print(filename, extension, typeDesc)


def writeFile(filepath, data, append=False, encoding="UTF-8"):
    """Writes the given data to the file at file path filename.

    If the file exists, the append argument determines whether or not it
    is overwritten (the default) or appended to. The data argument can
    be either a string or an array of bytes (commonly retrieved from a
    BLOB in a database or read from another file using
    system.file.readFileAsBytes).

    Args:
        filepath (str): The path of the file to write to.
        data (object): The character or binary content to write to the
            file.
        append (bool): If True(1), the file will be appended to if it
            already exists. If False(0), the file will be overwritten if
            it exists. The default is False(0). Optional.
        encoding (str): The character encoding of the file to write.
            Will throw an exception if the string does not represent a
            supported encoding. Common encodings are "UTF-8",
            "ISO-8859-1" and "US-ASCII". Default is "UTF-8". Optional.
    """
    print(filepath, data, append, encoding)



"""Date Functions.

The following functions give you access to test and modify dates.
"""

from __future__ import print_function

__all__ = [
    "addDays",
    "addHours",
    "addMillis",
    "addMinutes",
    "addMonths",
    "addSeconds",
    "addWeeks",
    "addYears",
    "daysBetween",
    "format",
    "fromMillis",
    "getAMorPM",
    "getDate",
    "getDayOfMonth",
    "getDayOfWeek",
    "getDayOfYear",
    "getHour12",
    "getHour24",
    "getMillis",
    "getMinute",
    "getMonth",
    "getQuarter",
    "getSecond",
    "getTimezone",
    "getTimezoneOffset",
    "getTimezoneRawOffset",
    "getYear",
    "hoursBetween",
    "isAfter",
    "isBefore",
    "isBetween",
    "isDaylightTime",
    "midnight",
    "millisBetween",
    "minutesBetween",
    "monthsBetween",
    "now",
    "parse",
    "secondsBetween",
    "setTime",
    "toMillis",
    "weeksBetween",
    "yearsBetween",
]

from calendar import monthrange
from datetime import datetime, timedelta
from time import localtime, mktime

from java.util import Date, Locale


def addDays(date, value):
    """Add or subtract an amount of days to a given date and time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date + timedelta(days=value)


def addHours(date, value):
    """Add or subtract an amount of hours to a given date and time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date + timedelta(hours=value)


def addMillis(date, value):
    """Add or subtract an amount of milliseconds to a given date and
    time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date + timedelta(milliseconds=value)


def addMinutes(date, value):
    """Add or subtract an amount of minutes to a given date and time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date + timedelta(minutes=value)


def addMonths(date, value):
    """Add or subtract an amount of months to a given date and time.

    This function is unique since each month can have a variable number
    of days. For example, if the date passed in is March 31st, and we
    add one month, April does not have a 31st day, so the returned date
    will be the proper number of months rounded down to the closest
    available day, in this case April 30th.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    month = (date.month + value) % 12
    year = date.year + (date.month + value - 1) // 12
    if not month:
        month = 12
    day = min(date.day, monthrange(year, month)[1])
    return date.replace(day=day, month=month, year=year)


def addSeconds(date, value):
    """Add or subtract an amount of seconds to a given date and time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date + timedelta(seconds=value)


def addWeeks(date, value):
    """Add or subtract an amount of weeks to a given date and time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date + timedelta(weeks=value)


def addYears(date, value):
    """Add or subtract an amount of years to a given date and time.

    Args:
        date (Date): The starting date.
        value (int): The number of units to add, or subtract if the
            value is negative.

    Returns:
        Date: A new date object offset by the integer passed to the
            function.
    """
    return date.replace(year=date.year + value)


def daysBetween(date_1, date_2):
    """Calculates the number of whole days between two dates.

    Daylight Saving Time changes are taken into account.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    return (date_2 - date_1).days


def format(date, format="yyyy-MM-dd HH:mm:ss"):
    """Returns the given date as a string, formatted according to a
    pattern.

    Note:
        Not all symbols from system.date.format() have a counterpart
        directive on strftime().

    Args:
        date (Date): The date to format.
        format (str): A format string such as "yyyy-MM-dd HH:mm:ss".
            Optional.

    Returns:
        str: A string representing the formatted datetime
    """
    _format = format.replace("yyyy", "%Y")
    _format = _format.replace("yy", "%y")
    _format = _format.replace("MMMM", "%B")
    _format = _format.replace("MMM", "%b")
    _format = _format.replace("MM", "%m")
    _format = _format.replace("dd", "%d")
    _format = _format.replace("HH", "%H")
    _format = _format.replace("hh", "%I")
    _format = _format.replace("mm", "%M")
    _format = _format.replace("S", "%f")
    _format = _format.replace("ss", "%S")
    _format = _format.replace("z", "%Z")
    _format = _format.replace("Z", "%z")
    _format = _format.replace("a", "%p")
    _format = _format.replace("w", "%U")
    _format = _format.replace("D", "%j")
    return date.strftime(_format)


def fromMillis(millis):
    """Creates a date object given a millisecond value.

    Args:
        millis (long): The number of milliseconds elapsed since
            January 1, 1970, 00:00:00 UTC (GMT).

    Returns:
        Date: A new date object.
    """
    seconds = millis // 1000
    micro = (millis % 1000) * 1000
    _date = datetime.fromtimestamp(seconds)
    return datetime(
        _date.year,
        _date.month,
        _date.day,
        _date.hour,
        _date.minute,
        _date.second,
        micro,
    )


def getAMorPM(date):
    """Returns a 0 if the time is before noon, and a 1 if the time is
    after noon.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return 1 if date.hour >= 12 else 0


def getDate(year, month, day):
    """Creates a new Date object given a year, month and a day.

    The time will be set to midnight of that day.

    Args:
        year (int): The year for the new date.
        month (int): The month of the new date. January is month 0.
        day (int): The day of the month for the new date. The first day
            of the month is day 1.

    Returns:
        Date: A new date, set to midnight of that day.
    """
    _jan = datetime(year, 1, day)
    return addMonths(_jan, month)


def getDayOfMonth(date):
    """Extracts the day of the month from a date. The first day of the
    month is day 1.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.day


def getDayOfWeek(date):
    """Extracts the day of the week from a date.

    Sunday is day 1, Saturday is day 7.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    _dow = [2, 3, 4, 5, 6, 7, 1]
    return _dow[date.weekday()]


def getDayOfYear(date):
    """Extracts the day of the year from a date. The first day of the
    year is day 1.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.timetuple().tm_yday


def getHour12(date):
    """Extracts the hour from a date. Uses a 12 hour clock, so noon and
    midnight are returned as 0.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.hour - 12 if date.hour >= 12 else date.hour


def getHour24(date):
    """Extracts the hour from a date. Uses a 24 hour clock, so midnight
    is zero.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.hour


def getMillis(date):
    """Extracts the milliseconds from a date, ranging from 0-999.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.microsecond // 1000


def getMinute(date):
    """Extracts the minutes from a date, ranging from 0-59.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.minute


def getMonth(date):
    """Extracts the month from a date, where January is month 0.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.month - 1


def getQuarter(date):
    """Extracts the quarter from a date, ranging from 1-4.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return ((date.month - 1) // 3) + 1


def getSecond(date):
    """Extracts the seconds from a date, ranging from 0-59.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.second


def getTimezone():
    """Returns the ID of the current timezone.

    Returns:
        str: A representation of the current timezone.
    """
    return "America/Tijuana"


def getTimezoneOffset(date=datetime.now()):
    """Returns the current timezone's offset versus UTC for a given
    instant, taking Daylight Saving Time into account.

    Args:
        date (Date): The instant in time for which to calculate the
            offset. Uses now() if omitted. Optional.

    Returns:
        float: The timezone offset compared to UTC, in hours.
    """
    return -7.0 if isDaylightTime(date) else -8.0


def getTimezoneRawOffset():
    """Returns the current timezone offset versus UTC, not taking
    Daylight Saving Time into account.

    Returns:
         float: The timezone offset.
    """
    offset = -1.0 if isDaylightTime() else 0.0
    return float(hoursBetween(datetime.utcnow(), datetime.now())) + offset


def getYear(date):
    """Extracts the year from a date.

    Args:
        date (Date): The date to use.

    Returns:
        int: An integer that is representative of the extracted value.
    """
    return date.year


def hoursBetween(date_1, date_2):
    """Calculates the number of whole hours between two dates.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    diff = date_2 - date_1
    days, seconds, _ = diff.days, diff.seconds, diff.microseconds
    return days * 24 + seconds // 3600


def isAfter(date_1, date_2):
    """Compares two dates to see if date_1 is after date_2.

    Args:
        date_1 (Date): The first date.
        date_2 (Date): The second date.

    Returns:
        bool: True (1) if date_1 is after date_2, False (0) otherwise.
    """
    return date_1 > date_2


def isBefore(date_1, date_2):
    """Compares to dates to see if date_1 is before date_2.

    Args:
        date_1 (Date): The first date.
        date_2 (Date): The second date.

    Returns:
        bool: True (1) if date_1 is before date_2, False (0) otherwise.
    """
    return date_1 < date_2


def isBetween(target_date, start_date, end_date):
    """Compares two dates to see if a target date is between two other
    dates.

    Args:
        target_date (Date): The date to compare.
        start_date (Date): The start of a date range.
        end_date (Date): The end of a date range. This date must be
            after the start date.

    Returns:
        bool: True (1) if target_date is >= start_date and
            target_date <= end_date, False (0) otherwise.
    """
    return start_date <= target_date <= end_date


def isDaylightTime(date=datetime.now()):
    """Checks to see if the current timezone is using Daylight Saving
    Time during the date specified.

    Args:
        date (Date): The date you want to check if the current
            timezone is observing Daylight Saving Time. Uses now() if
            omitted. Optional.

    Returns:
        bool: True (1) if date is observing Daylight Saving Time in the
            current timezone, False (0) otherwise.
    """
    time_tuple = (
        date.year,
        date.month,
        date.day,
        date.hour,
        date.minute,
        date.second,
        date.weekday(),
        0,
        0,
    )
    stamp = mktime(time_tuple)
    time_tuple = localtime(stamp)
    return time_tuple.tm_isdst > 0


def midnight(date):
    """Returns a copy of a date with the hour, minute, second, and
    millisecond fields set to zero.

    Args:
        date (Date): The starting date.

    Returns:
        Date: A new date, set to midnight of the day provided.
    """
    return date.replace(hour=0, minute=0, second=0, microsecond=0)


def millisBetween(date_1, date_2):
    """Calculates the number of whole milliseconds between two dates.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        long: An integer that is representative of the difference
            between two dates.
    """
    diff = date_2 - date_1
    days, seconds, microseconds = diff.days, diff.seconds, diff.microseconds
    return days * 86400 * 1000 + seconds * 1000 + microseconds // 1000


def minutesBetween(date_1, date_2):
    """Calculates the number of whole minutes between two dates.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    diff = date_2 - date_1
    days, seconds, _ = diff.days, diff.seconds, diff.microseconds
    return days * 1440 + seconds // 60


def monthsBetween(date_1, date_2):
    """Calculates the number of whole months between two dates. Daylight
    Saving Time changes are taken into account.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    delta = 0
    while True:
        mdays = monthrange(date_1.year, date_2.month)[1]
        date_1 += timedelta(days=mdays)
        if date_1 <= date_2:
            delta += 1
        else:
            break
    return delta


def now():
    """Returns a java.util.Date object that represents the current time
    according to the local system clock.

    Returns:
        Date: A new date, set to the current date and time.
    """
    return datetime.now()


def parse(
    dateString, formatString="yyyy-MM-dd HH:mm:ss", locale=Locale.ENGLISH
):
    """Attempts to parse a string and create a Date. Causes
    ParseException if the date dateString parameter is in an
    unrecognized format.

    Args:
        dateString (str): The string to parse into a date.
        formatString (str): Format string used by the parser. Default is
            "yyyy-MM-dd HH:mm:ss". Optional.
        locale (object): Locale used for parsing. Can be the locale name
            such as 'fr', or the Java Locale such as 'Locale.French'.
            Default is 'Locale.English'. Optional.

    Returns:
        Date: The parsed date.
    """
    print(dateString, formatString, locale)
    return now()


def secondsBetween(date_1, date_2):
    """Calculates the number of whole seconds between two dates.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    diff = date_2 - date_1
    days, seconds, _ = diff.days, diff.seconds, diff.microseconds
    return days * 86400 + seconds


def setTime(date, hour, minute, second):
    """Takes in a date, and returns a copy of it with the time fields
    set as specified.

    Args:
        date (Date): The starting date.
        hour (int): The hours (0-23) to set.
        minute(int): The minutes (0-59) to set.
        second (int): The seconds (0-59) to set.

    Returns:
        Date: A new date, set to the appropriate time.
    """
    return date.replace(hour=hour, minute=minute, second=second, microsecond=0)


def toMillis(date):
    """Converts a Date object to its millisecond value elapsed since
    January 1, 1970, 00:00:00 UTC (GMT).

    Args:
        date (Date): The date object to convert.

    Returns:
        int: 8-byte integer representing the number of millisecond
            elapsed since January 1, 1970, 00:00:00 UTC (GMT).
    """
    millis = mktime(date.timetuple()) * 1000 + date.microsecond // 1000
    return int(millis)


def weeksBetween(date_1, date_2):
    """Calculates the number of whole weeks between two dates.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    diff = date_2 - date_1
    return diff.days // 7


def yearsBetween(date_1, date_2):
    """Calculates the number of whole years between two dates. Daylight
    Saving Time changes are taken into account.

    Args:
        date_1 (Date): The first date to use.
        date_2 (Date): The second date to use.

    Returns:
        int: An integer that is representative of the difference between
            two dates.
    """
    return monthsBetween(date_1, date_2) // 12



"""Navigation Functions.

The following functions allow you to open and close windows in the
client.
"""

from __future__ import print_function

__all__ = [
    "centerWindow",
    "closeParentWindow",
    "closeWindow",
    "desktop",
    "getCurrentWindow",
    "goBack",
    "goForward",
    "goHome",
    "openWindow",
    "openWindowInstance",
    "swapTo",
    "swapWindow",
]

from com.inductiveautomation.factorypmi.application import FPMIWindow
from com.inductiveautomation.factorypmi.application.script.builtin import (
    NavUtilities,
)
from java.util import EventObject


def centerWindow(arg):
    """Given a window path, or a reference to a window itself, it will
    center the window.

    The window should be floating and non-maximized. If the window can't
    be found, this function will do nothing.

    Args:
        arg (object): The path of the window (str) or a reference to the
            window (FPMIWindow) to center.
    """
    print(arg)


def closeParentWindow(event):
    """Closes the parent window given a component event object.

    Args:
        event (EventObject): A component event object. The enclosing
            window for the component will be closed.
    """
    print(event)


def closeWindow(arg):
    """Given a window path, or a reference to a window itself, it will
    close the window.

    If the window can't be found, this function will do nothing.

    Args:
        arg (object): A reference to the window to close as an
            FPMIWindow instance or the path of the window to close as a
            String.
    """
    print(arg)


def desktop(handle="primary"):
    """Allows for invoking system.nav functions on a specific desktop.

    Args:
        handle (str): The handle for the desktop to use. The screen
            index casted as a string may be used instead of the handle.
            If omitted, this will default to the Primary Desktop.
            Alternatively, the handle "primary" can be used to refer to
            the Primary Desktop.

    Returns:
        NavUtilities: A copy of system.nav that will alter the desktop
            named by the given handle.
    """
    print(handle)
    return NavUtilities()


def getCurrentWindow():
    """Returns the path of the current "main screen" window, which is
    defined as the maximized window.

    With the Typical Navigation Strategy, there is only ever one
    maximized window at a time.

    Returns:
        str: The path of the current "main screen" window - the
            maximized window.
    """
    return "Path/To/Maximized Window"


def goBack():
    """When using the Typical Navigation Strategy, this function will
    navigate back to the previous main screen window.

    Returns:
        FPMIWindow: The window that was returned to.
    """
    return FPMIWindow("Back")


def goForward():
    """When using the Typical Navigation Strategy, this function will
    navigate "forward" to the last main-screen window the user was on
    when they executed a system.nav.goBack().

    Returns:
        FPMIWindow: The window that was returned to.
    """
    return FPMIWindow("Forward")


def goHome():
    """When using the Typical Navigation Strategy, this function will
    navigate to the "home" window.

    This is automatically detected as the first main-screen window shown
    in a project.

    Returns:
        FPMIWindow: The window that was returned to.
    """
    return FPMIWindow("Home")


def openWindow(path, params=None):
    """Opens the window with the given path.

    If the window is already open, brings it to the front. The optional
    params dictionary contains key:value pairs which will be used to set
    the target window's root container's dynamic variables.

    Args:
        path (str): The path to the window to open.
        params (dict): A dictionary of parameters to pass into the
            window. The keys in the dictionary must match dynamic
            property names on the target window's root container. The
            values for each key will be used to set those properties.
            Optional.

    Returns:
        FPMIWindow: A reference to the opened window.
    """
    print(path, params)
    return FPMIWindow("Opened Window")


def openWindowInstance(path, params=None):
    """Operates exactly like system.nav.openWindow, except that if the
    named window is already open, then an additional instance of the
    window will be opened.

    There is no limit to the number of additional instances of a window
    that you can open.

    Args:
        path (str): The path to the window to open.
        params (dict): A dictionary of parameters to pass into the
            window. The keys in the dictionary must match dynamic
            property names on the target window's root container. The
            values for each key will be used to set those properties.
            Optional.

    Returns:
        FPMIWindow: A reference to the opened window.
    """
    print(path, params)
    return FPMIWindow("Window Instance")


def swapTo(path, params=None):
    """Performs a window swap from the current main screen window to the
    window specified.

    Swapping means that the opened window will take the place of the
    closing window - in this case it will be maximized.

    This function works like system.nav.swapWindow except that you
    cannot specify the source for the swap.

    Args:
        path (str): The path to the window to open.
        params (dict): A dictionary of parameters to pass into the
            window. The keys in the dictionary must match dynamic
            property names on the target window's root container. The
            values for each key will be used to set those properties.
            Optional.

    Returns:
        FPMIWindow: A reference to the swapped-to window.
    """
    print(path, params)
    return FPMIWindow("Swapped To")


def swapWindow(arg, swapToPath, params=None):
    """Performs a window swap.

    This means that one window is closed, and another is opened and
    takes its place - assuming its size, floating state, and
    maximization state. This gives a seamless transition; one window
    seems to simply turn into another.

    Args:
        arg (object): The path of the window (str) to swap from. Must be
            a currently open window, or this will act like an
            openWindow, or a component event (EventObject) whose
            enclosing window will be used as the "swap-from" window.
        swapToPath (str): The name of the window to swap to.
        params (dict): A dictionary of parameters to pass into the
            window. The keys in the dictionary must match dynamic
            property names on the target window's root container. The
            values for each key will be used to set those properties.
            Optional.

    Returns:
        FPMIWindow: A reference to the swapped-to window.
    """
    print(arg, swapToPath, params)
    return FPMIWindow("Swapped To")
